

################################################################################
# UNICODE
#
#     \u03c6 \u03d5 \u03a6 (variations of greek phi) the last, the best
#     \u221A square root
#     \u221B cubic root
#     \u222B integral
#     \u2211 sumatori
#     \u2264 less or equal
#     \u2265 greater or equal
#     \u2270 NO less or equal
#     \u2271 NO grater or equal


namespace eval Concreter {
    variable nodes  ;# coordinates list
    variable trielms ;# connectivities for triangles
    variable linelms ;# connectivities for line elements
    variable trinodeselms ;# triangles around node
    variable linenodeselms   ;# line elms around node
    variable trielmsgroup ;# groups of triangle elements that are together
    variable trielmsedges ;# boundaries for the groups
    variable trielmbeamnodes ;# nodes that have line elements for every group

    variable canvas
    variable status
    variable progressvar
}

proc Concreter::Init { { w .tc } } {
    variable canvas

    set ::math::epsilon .1

    catch { destroy $w }
    toplevel $w
    wm title $w "Dimension concrete"

    set descmenu [list \
	"&File" all file 0 [list \
	    [list command "&Output SVG" {} "Output SVG" "" \
		-command "Concreter::OutputSVG $w"] \
	    separator \
	    [list command "&Close" {} "Exit program" "Ctrl w" \
		-command "destroy $w"] \
	   ] \
	"&Draw" all draw 0 [list \
	    [list command "&Init Mesh" {} "Init mesh" "" \
		-command "Concreter::InitMeshes"] \
	    [list command "&Draw map" {} "Draw map" "Ctrl d" \
		-command "Concreter::DrawMap $w 1"] \
	   ] \
       ]
    set mainframe [g_MainFrame $w.mainframe \
	-textvariable Concreter::status \
	-progressvar Concreter::progressvar -progressmax 100 \
	-progresstype normal -menu $descmenu -grid 0]
    $mainframe showstatusbar progression 
    set f [$mainframe getframe]

    set sw [g_ScrolledWindow $f.lf -relief sunken -borderwidth 0 -grid 0]
    set canvas($w) [canvas $sw.c -bg white -width 1200 -height 800]
    $sw setwidget $canvas($w)

    supergrid::go $f
    supergrid::go $w

    catch { font create CanvasNormalFont -family {MS Sans Serif} -size 8}
    ApplyCanvasBindings $canvas($w)
}

proc Concreter::EnterLeaveCanvasItem { can item what { x "" } { y "" } } {
    variable trinodeselms
    variable nodes

    set tags [$can gettags $item]

    if { [lsearch $tags boundary] != -1 } {
	set maintag boundary
    } else {
	set ipos [lsearch -regexp $tags {n[0-9]+}]
	set maintag [lindex $tags $ipos]
    }
    switch $what {
	Enter {
	    catch { unset ::savefills }
	    foreach i [$can find withtag $maintag] {
		set ::savefills($i) [$can itemcget $i -fill]
		$can itemconf $i -fill red
	    }
	    
	}
	Leave {
	    foreach i [$can find withtag $maintag] {
		$can itemconf $i -fill $::savefills($i)
	    }
	}
	ButtonRelease-3 {
	    regexp {n([0-9]+)} $maintag {} nod
	    set trielm [lindex $trinodeselms($nod) 0]
	    set pointelem $nodes($nod)
	    
	    catch { destroy $can.menu }
	    menu $can.menu
	    $can.menu add command -label "Calculate" -command \
		[list Concreter::CalcSteelSecForElem $trielm $pointelem 1 "" 1]
	    tk_popup $can.menu $x $y
	}
    }
}

proc Concreter::ApplyCanvasBindings { can } {

    bind $can <2> {
	%W scan mark %x %y
	set tkPriv(x) %x
	set tkPriv(y) %y
	set tkPriv(mouseMoved) 0
    }
    bind $can <B2-Motion> {
	if {(%x != $tkPriv(x)) || (%y != $tkPriv(y))} {
	    set tkPriv(mouseMoved) 1
	}
	if {$tkPriv(mouseMoved)} {
	    %W scan dragto %x %y
	}
    }

    $can bind all <Enter> "Concreter::EnterLeaveCanvasItem $can current Enter"
    $can bind all <Leave> "Concreter::EnterLeaveCanvasItem $can current Leave"
    $can bind all <ButtonRelease-3> "Concreter::EnterLeaveCanvasItem $can current ButtonRelease-3 %X %Y"
 
#     bind $can <1> ".tc.t ins end \"\[$can canvasx %x] \[$can canvasy %y]\n\""

}

proc Concreter::WaitState { what { w . } } {
    if { $what == 1 } {
	$w configure -cursor watch
	if { [winfo toplevel $w] != $w } {
	    [winfo toplevel $w] configure -cursor watch
	}
    } else {
	$w configure -cursor ""
	if { [winfo toplevel $w] != $w } {
	    [winfo toplevel $w] configure -cursor ""
	}
    }
    update
}

proc Concreter::ProgressVar { value { canstop 0 } } {
    variable progressvar
    variable text

    if { [info exists progressvar] && $progressvar == -2 } {
	set Concreter::progressvar -1
	if { [winfo exists $text.__frame] } { destroy $text.__frame }
	bind all <Escape> ""
	error "Stop at user demand"
    }

    set progressvar $value

    if { $canstop == 1 && $value == 0 } {
	if { [winfo exists $text.__frame] } { destroy $text.__frame }
	frame $text.__frame
	focus $text.__frame
	catch { grab $text.__frame }
	bind all <Escape> "set Concreter::progressvar -2"
    }

    if { $value == 100 } {
	after 1000 set Concreter::progressvar -1
	if { [winfo exists $text.__frame] } { destroy $text.__frame }
	bind all <Escape> ""
    }
    update
}

proc Concreter::OutputSVG { w } {
    variable canvas

    source /tcltk/can2svg.tcl
    ::can2svg::canvas2file $canvas($w) /temp/kk.svg -width 1200 -height 800 \
	-fontmap [list CanvasNormalFont "Arial 8"]
    eval exec [auto_execok start] [list [file native /temp/kk.svg]] &
}

proc Concreter::InitMeshes {} {
    variable nodes
    variable trielms
    variable linelms   
    variable trinodeselms
    variable linenodeselms   
    variable trielmsgroup
    variable trielmsedges
    variable trielmbeamnodes

    WaitState 1

    set vars [list trielms linelms trinodeselms linenodeselms trielmsgroup \
	    trielmsedges trielmbeamnodes]
    foreach i $vars { catch { unset $i } }


    foreach "num x y z" [.central.s info Mesh Nodes] {
	set nodes($num) [list $x $y $z]
    }
    foreach "num n1 n2 n3 mat" [.central.s info Mesh Elements Triangle] {
	set trielms($num) [list $n1 $n2 $n3 $mat 0]
	lappend trinodeselms($n1) $num
	lappend trinodeselms($n2) $num
	lappend trinodeselms($n3) $num        
    }
    foreach "num n1 n2 mat" [.central.s info Mesh Elements Linear] {
	set zdiff [expr {[lindex $nodes($n1) 2]-[lindex $nodes($n2) 2]}]
	if { abs($zdiff) < $::math::epsilon } { continue }
	set linelms($num) [list $n1 $n2 $mat]
	lappend linenodeselms($n1) $num
	lappend linenodeselms($n2) $num
    }

    set ig 1
    set trielmsgroup($ig) [lindex [array names trielms] 0]
    set trielmbeamnodes($ig) ""
    set curr 0
    set currnum [lindex $trielmsgroup($ig) $curr]
    set trielms($currnum) [lreplace $trielms($currnum) 4 4 $ig]

    set len 1
    while 1 {
	set n1 [lindex $trielms($currnum) 0]
	for { set i 0 } { $i < 3 } { incr i } {
	    set n2 [lindex $trielms($currnum) [expr {($i<2)?$i+1:0}]]
	    set face [GiveBoundaryTri $currnum $n1 $n2]
	    if { $face == "" } {
		set edges($n1) $n2
	    } else {
		if { [lindex $trielms($face) 4] == 0 } {
		    lappend trielmsgroup($ig) $face
		    incr len
		    set trielms($face) [lreplace $trielms($face) 4 4 $ig]
		}
	    }
	    if { [info exists linenodeselms($n1)] } {
		set beamnodes($n1) 1
	    }
	    set n1 $n2
	}
	incr curr
	if { $curr >= $len } { break }
	set currnum [lindex $trielmsgroup($ig) $curr]
    }
    set trielmbeamnodes($ig) [array names beamnodes]
    unset beamnodes

    set icurve 1
    set currnod [lindex [array names edges] 0]
    set trielmsedges($ig,$icurve) $currnod
    while 1 {
	if { ![info exists edges($currnod)] } {
	    incr icurve
	    set currnod [lindex [array names edges] 0]
	    if { $currnod == "" } { break }
	    set trielmsedges($ig,$icurve) $currnod
	} else {
	    set currnod_new $edges($currnod)
	    lappend trielmsedges($ig,$icurve) $currnod_new
	    unset edges($currnod)
	    set currnod $currnod_new
	}
    }
    incr icurve -1
    unset edges

#     puts trielmbeamnodes($ig)=$trielmbeamnodes($ig)
#     for { set i 1 } { $i <= $icurve } { incr i } {
#         puts i=$i---$trielmsedges($ig,$i)
#     }
 
#     .central.s process escape escape escape escape
#     if { [lindex [.central.s info Project] 3] == "GEOMETRYUSE" } {
#         .central.s process Meshing MeshView
#     }
    #eval .central.s process SelectEntities Elements AddToSelection $trielmsgroup($ig)
    #eval .central.s process SelectEntities Nodes AddToSelection $trielmsedges($ig)
    #eval .central.s process SelectEntities Nodes AddToSelection $trielmsedges($ig,1)

     WaitState 0
}

proc Concreter::GiveBoundaryTri { trinum n1 n2 } {
    variable trielms
    variable trinodeselms
	
    foreach elm1 $trinodeselms($n1) {
	if { $elm1 == $trinum } { continue }
	foreach elm2 $trinodeselms($n2) {
	    if { $elm1 == $elm2 } {
		return $elm1
	    }
	} 
    }
    return ""
}

proc Concreter::SquareCoords { x y wx wy } {
    return [list [expr {$x-$wx/2.0}] [expr {$y-$wy/2.0}] \
	    [expr {$x-$wx/2.0}] [expr {$y+$wy/2.0}] \
	    [expr {$x+$wx/2.0}] [expr {$y+$wy/2.0}] \
	    [expr {$x+$wx/2.0}] [expr {$y-$wy/2.0}] \
	    [expr {$x-$wx/2.0}] [expr {$y-$wy/2.0}]]
}

proc Concreter::DrawMap { w ig } {
    variable nodes
    variable trielmsedges
    variable trielmbeamnodes
    variable linenodeselms
    variable linelms
    variable trinodeselms
    variable trielms
    variable canvas

    set can $canvas($w)
    if { ![info exists trielmbeamnodes($ig)] } {
	WarnWin "Cannot draw map. Select one group first" $can
	return
    }

    WaitState 1


    $can delete all

    set xpsense 1

    foreach "xmin xmax ymin ymax" [list 1e10 -1e10 1e10 -1e10] break

    foreach name [array names trielmsedges $ig,*] {
	set n1 [lindex $trielmsedges($name) 0]
	foreach n2 [lrange $trielmsedges($name) 1 end] {
	    foreach "x1 y1" $nodes($n1) break 
	    foreach "x2 y2" $nodes($n2) break 
	    $can create line $x1 $y1 $x2 $y2 -fill black -tags boundary

	    if { $x1 < $xmin } { set xmin $x1 }
	    if { $x1 > $xmax } { set xmax $x1 }
	    if { $y1 < $ymin } { set ymin $y1 }
	    if { $y1 > $ymax } { set ymax $y1 }

	    set n1 $n2
	}
    }
    set sectionbase ""
    foreach i $trielmbeamnodes($ig) {
	foreach "x y" $nodes($i) break

	set err 1
	foreach elm $linenodeselms($i) {
	    set err [catch { 
		    foreach "mat widthY widthZ concrete_res" [GiveSectionProps $elm] break
		} errstring]
	    if { !$err } { break }
	}
	if { $err } { continue }
	
	if { abs([lindex $mat 4]) < $math::epsilon } {
	    # jprima points to x global
	    $can create line [SquareCoords $x $y $widthY $widthZ] -fill blue -tags "support n$i"
	} else {
	    # nothing by now
	}
	set trielm [lindex $trinodeselms($i) 0]
	foreach "trimat - - -" [GiveSectionProps $elm] break
	#set pointelem [join $nodes($i) ,]
	set pointelem $nodes($i)

	if { $sectionbase == "" } {
	    set sectionbase [CalcSteelSecForElemMinimum $trielm $pointelem $xpsense]
	}

	set section [CalcSteelSecForElem $trielm $pointelem $xpsense $sectionbase]

	set negative ""
	set positive ""
	foreach j $trielmbeamnodes($ig) {
	    if { $i == $j } { continue }
	    foreach "x2 y2" $nodes($j) break
	    set r [expr {hypot($x2-$x,$y2-$y)}]
	    set phi [expr {atan2(($y2-$y),($x2-$x))}]

	    if { !$xpsense } {
		set phi [expr {$phi-0.5*$math::PI}]
	    }

	    if { abs($phi) < 0.2*$math::PI } {
		if { $positive == "" || [lindex $positive 1] > $r } {
		    set positive [list $j $r]
		}
	    } elseif { abs($math::PI-$phi) < 0.2*$math::PI } {
		if { $negative == "" || [lindex $negative 1] > $r } {
		    set negative [list $j $r]
		}
	    }
	}
	if { $positive != "" } {
	    Concreter::DrawSteel $can $x $y [expr {[lindex $positive 1]/4.0}] positive $section \
	    $xpsense n$i
	    if { $negative == "" } {
		Concreter::DrawSteel $can $x $y 0 negative "" $xpsense \
		    n$i
	    }
	}
	if { $negative != "" } {
	    if { $positive == "" } {
		set tt $section
	    } else { set tt "" }
	    Concreter::DrawSteel $can $x $y [expr {[lindex $negative 1]/4.0}] negative $tt $xpsense \
		n$i
	    if { $positive == "" } {
		Concreter::DrawSteel $can $x $y 0 positive "" $xpsense \
		    n$i
	    }
	}
    }

    set deltax [expr {$xmax-$xmin}]
    set xmin [expr {$xmin-0.05*$deltax}]
    set xmax [expr {$xmax+0.05*$deltax}]
    set ymin [expr {$ymin-0.05*$deltax}]
    set ymax [expr {$ymax+0.05*$deltax}]

    set factor [expr {1000.0/$deltax}]
    set factory [expr {-1*$factor}]
     
    $can scale all 0 0 $factor $factory
    $can conf -scrollregion [list [expr {$xmin*$factor}] [expr {-$ymax*$factor}] \
	    [expr {$xmax*$factor}] [expr {-$ymin*$factor}]]

    WaitState 0
}

proc Concreter::DrawSteel { can x y l posneg text xpsense tags } {

    # \u03c6---\u03d5
    regsub -all D $text \u03a6 text
    switch $xpsense {
	1 {
	    switch $posneg {
		positive { set vec [list 1 0] }
		negative { set vec [list -1 0] }
	    }
	    if { $l == 0 } {
		set l .1
		set vec2 [list 0 -.1]
	    } else {
		set vec2 [list [expr {.1*[lindex $vec 0]}] -.1]
	    }
	}
	0 {
	    switch $posneg {
		positive { set vec [list 0 1] }
		negative { set vec [list 0 -1] }
	    }
	    if { $l == 0 } {
		set vec2 [list -.1 0]
	    } else {
		set vec2 [list -.1 [expr {.1*[lindex $vec 1]}]]
	    }
	}
    }
    $can create line $x $y [expr {$x+[lindex $vec 0]*$l}] [expr {$y+[lindex $vec 1]*$l}] \
	[expr {$x+[lindex $vec 0]*$l+[lindex $vec2 0]}] \
	[expr {$y+[lindex $vec 1]*$l+[lindex $vec2 1]}] -width 2 -tags [concat steel $tags]
    if { $text != "" } {
	$can create text [expr {$x+.1}] [expr {$y+.1}] -text $text -anchor sw -tags \
	    [concat steel $tags] -font CanvasNormalFont
    }

}

proc Concreter::CalcSteelSecForElemMinimum { elm pointelem xpsense } {
    variable prefsinit
    variable nodes
    variable trielms

    if { ![info exists prefsinit] } {
	set prefsinit 1
	ConcretePrefs::init $::ProblemTypePriv(problemtypedir)
    }
    set err [catch { GiveSectionProps $elm } retval]
    if { $err } { error $retval }
    foreach "mat widthY widthZ concrete_res" $retval break

    set newsec [dimensioner::DimensionForPlateShell 0.0 $widthY $widthZ $concrete_res ""]

    return $newsec
}

proc Concreter::CalcSteelSecForElem { elm pointelem xpsense basesec { dowrite 0 } } {
    variable prefsinit
    variable nodes
    variable trielms

    if { ![info exists prefsinit] } {
	set prefsinit 1
	ConcretePrefs::init $::ProblemTypePriv(problemtypedir)
    }
    set err [catch { GiveSectionProps $elm } retval]
    if { $err } { error $retval }
    foreach "mat widthY widthZ concrete_res" $retval break

    catch { unset Results }
    array set Results [Concreter::GiveArrayResultsForElm $elm]

    set Mmax 0.0
    foreach i [array names Results *,Momentus] {
	if { $xpsense } {
	    set Mgauss [list [lindex $Results($i) 1] [lindex $Results($i) 6] [lindex $Results($i) 11]]
	} else {
	    set Mgauss [list [lindex $Results($i) 2] [lindex $Results($i) 7] [lindex $Results($i) 12]]
	}
	set err [catch { math::CalcValueInTrianglePoint $elm nodes trielms $pointelem $Mgauss } M]
	if { abs($M) > abs($Mmax) } { set Mmax $M }
    }
    set newsec [dimensioner::DimensionForPlateShell $Mmax $widthY $widthZ $concrete_res \
	$basesec $dowrite]
    return $newsec
}

# proc Concreter::CalcSteelSecForElemOld { elm pointelem xpsense basesec } {
#     variable prefsinit
#     variable trielms
#     variable nodes
#     variable pipeid

#     if { ![info exists prefsinit] } {
# 	set prefsinit 1
# 	ConcretePrefs::init $::ProblemTypePriv(problemtypedir)
#     }
#     set err [catch { GiveSectionProps $elm } retval]
#     if { $err } { error $retval }
#     foreach "mat widthY widthZ concrete_res" $retval break

#     set err [catch { Concreter::GiveResultsForElm $elm } retresults]
#     if { $err } { error $retresults }

#     set err [catch {ConcretePrefs::GivePrefsDataForDimension} retprefsdata]
#     if { $err } { error $retprefsdata }

#     set data ""
#     append data $retprefsdata

#     set ipos [lsearch [.central.s info gendata] Mesh_units*]
#     set meshunits [lindex [.central.s info gendata] [expr $ipos+1]]
#     set fac [ConvertLengthsUnitsToSI $meshunits]

#     for { set i 0 } { $i < 3 } { incr i } {
# 	set n [lindex $trielms($elm) $i]
# 	append data "Node $n "
# 	foreach j $nodes($n) {
# 	    append data "[expr {$fac*$j}] "
# 	}
# 	append data \n
#     }
#     append data "element $elm [lrange $trielms($elm) 0 2]\n"
#     regexp {([^,]+),([^,]+),([^,]+)} $pointelem {} x y z
#     set pointelemL "[expr $fac*$x],[expr $fac*$y],[expr $fac*$z]\n"
#     append data "PointInElement $pointelemL\n"

#     append data "widthY $widthY\n"
#     append data "widthZ $widthZ\n"
#     append data "concrete_res $concrete_res\n"

#     if { $xpsense } {
# 	append data "shell_element xpsense\n"
#     } else {
# 	append data "shell_element ypsense\n"
#     }
#     if { $basesec != "" } {
# 	append data $retresults
# 	append data "Section $basesec\n"
#     }
#     append data enddata

#     if { $pipeid == "" } {
# 	set exec /Users/ramsan/barres/barres/release/barres.exe
# 	if { ![file exists $exec] } {
# 	    set exec /home/ramsan/barreswin/barres_release/barres
# 	}
# 	if { ![file exists $exec] } {
# 	    error "could not find concrete executable"
# 	}
# 	set pipeid [open "|[list $exec -c -]" w+]
# 	fconfigure $pipeid -blocking 0
#     }
#     fileevent $pipeid readable "set ::pipeidread 1; fileevent $pipeid readable {}"
#     set ::pipeidread 0

#     set err [catch {
# 	puts $pipeid $data
# 	flush $pipeid
#     } errstring]
#     if { $err } {
# 	set ::pipeidread 1
#     }
	
#     if { $::pipeidread == 0 } {
# 	vwait ::pipeidread
#     }
#     set retval [read $pipeid]

#     if { $basesec == "" } {
# 	regexp {Section\s+(.*)} $retval {} sec
# 	return $sec
#     }

#     set As ""; set Aps ""
#     regexp {Section\s+As=(\S+)\s+A's=(\S)} $retval {} As Aps
#     return [list $As $Aps]
# }

proc Concreter::GiveSectionProps { elm } {
    variable nodes
    variable linelms
    variable trielms

    set type ""
    foreach i "Rectangular_Section Section Concrete_rec_section Shell Concrete_Shell \
		    Anisotropic_Shell" {
	set retval [lindex [.central.s info conditions -localaxesmat $i mesh $elm] 0]
	if { [lindex $retval 0] != 0 } {
	    set type $i
	    break
	}
    }
    switch $type {
	Section {
	    error "Section props unknown"
	}
	Concrete_rec_section {
	    foreach "- - - mat units widthY widthZ concrete_res" $retval break
	    if { $mat == "-Default-" } {
		set mat [math::BeamTransfMatrix [lrange $linelms($elm) 0 1] nodes]
	    }
	    set fac [ConvertUnitsToSI $units m]
	    set widthY [expr $fac*$widthY]
	    set widthZ [expr $fac*$widthZ]
	    return [list $mat $widthY $widthZ $concrete_res]
	}
	Rectangular_Section {
	    error "concrete props unknown"
	}
	Concrete_Shell {
	    foreach "- - - mat units width concrete_res" $retval break
	    if { $mat == "-Default-" } {
		set mat [math::ShellTransfMatrix [lrange $trielms($elm) 0 2] nodes]
	    }
	    set fac [ConvertUnitsToSI $units m]
	    set widthY 1.0
	    set widthZ [expr $fac*$width]
	     return [list $mat $widthY $widthZ $concrete_res]
	}
	Shell {
	    error "concrete props unknown"
	}
	default {
	    error "Properties type '$type' not implemented"
	}
    }
}

proc Concreter::GiveArrayResultsForElm { elem } {

    catch { unset Results }
    
    set val [.central.s info list_entities Elements $elem]
    if { [lindex $val 0] == "Bad" } {
	error "Problems obtaining elements results"
    }
    
    set nodes [lindex $val 3]
    set elemtype [lindex $val 4]
    set nres [lindex [lindex $val 8] 1]
    set allres [lindex $val 12]
    
    set validres { {"B Axial Force" N} {BDisplacements m} {Torsor Nm} {"Y' Momentus" Nm} \
	{"Y' Shear" N} {"Z' Momentus" Nm} {"Z' Shear" N} {"Local Axes Beam" ""} \
	{"Axial Force" N/m} {Momentus N } {Shear N/m} {"Local Axes" ""} }
    
    set curr [.central.s info postprocess get cur_analisis]
    set alla [.central.s info postprocess get all_analisis]
    
    set ipos [lsearch $alla Max_Min]
    if { $ipos != -1 } {
	set alla [lreplace $alla $ipos $ipos]
    }
    
    set ipos [lsearch [.central.s info gendata] Results_units*]
    set units [lindex [.central.s info gendata] [expr $ipos+1]]
    
    foreach k $alla {
	.central.s process escape escape escape escape Results AnalysisSel $k \
	    escape escape
	set val [.central.s info list_entities Elements $elem]
	set pos [lsearch $val More]
	incr pos
	incr pos
	set allres [lindex $val $pos]
	foreach i $allres {
	    set resname [lindex $i 0]
	    regexp {^(.*)[ ]+[(][^\)]*[)]$} $resname {} resname
	    set found 0
	    set fac ""
	    foreach r $validres {
		if { $resname == [lindex $r 0] } {
		    if { [lindex $r 1] == "" } {
		        set fac 1.0
		    } else {
		        set fac [ConvertUnitsToSI $units [lindex $r 1]]
		    }
		    set found 1
		    break
		}
	    }
	    if { !$found } { continue }
	    if { $fac == "" } { return }
	    set type [lindex $i 1]
	    set Results($k,$resname) ""
	    switch $type {
		Beams {
		    foreach j [lindex $i 3] {
		        foreach l [lindex $j 3] {
		            lappend Results($k,$resname) [expr $fac*$l]
		        }
		    }                                     
		}                                     
		Beams1 {                             
		    foreach j [lindex $i 3] {             
		        lappend Results($k,$resname) [lindex $j 0]
		        lappend Results($k,$resname) [lindex $j 1]
		        lappend Results($k,$resname) [lindex $j 2]
		    }
		}
		Triangles {
		    set inum 1
		    foreach j [lindex $i 3] {
		        append Results($k,$resname) " GP$inum"
		        foreach l $j {
		            lappend Results($k,$resname) [expr $fac*$l]
		        }
		        incr inum
		    }
		}
		Triangles1 {
		    foreach j [lindex $i 3] {
		        lappend Results($k,$resname) [lindex $j 0]
		        lappend Results($k,$resname) [lindex $j 1]
		        lappend Results($k,$resname) [lindex $j 2]
		    }
		}
	    }
	}
    }
    .central.s process escape escape escape escape Results AnalysisSel $curr \
	escape escape

    return [array get Results]
}

proc Concreter::GiveResultsForElm { elem } {

    catch { unset Results }
    
    set val [.central.s info list_entities Elements $elem]
    if { [lindex $val 0] == "Bad" } {
	error "Problems obtaining elements results"
    }
    
    set nodes [lindex $val 3]
    set elemtype [lindex $val 4]
    set nres [lindex [lindex $val 8] 1]
    set allres [lindex $val 12]
    
    set validres { {"B Axial Force" N} {BDisplacements m} {Torsor Nm} {"Y' Momentus" Nm} \
	{"Y' Shear" N} {"Z' Momentus" Nm} {"Z' Shear" N} {"Local Axes Beam" ""} \
	{"Axial Force" N/m} {Momentus N } {Shear N/m} {"Local Axes" ""} }
    
    set curr [.central.s info postprocess get cur_analisis]
    set alla [.central.s info postprocess get all_analisis]
    
    set ipos [lsearch $alla Max_Min]
    if { $ipos != -1 } {
	set alla [lreplace $alla $ipos $ipos]
    }
    
    set ipos [lsearch [.central.s info gendata] Results_units*]
    set units [lindex [.central.s info gendata] [expr $ipos+1]]
    
    foreach k $alla {
	.central.s process escape escape escape escape Results AnalysisSel $k \
	    escape escape
	set val [.central.s info list_entities Elements $elem]
	set pos [lsearch $val More]
	incr pos
	incr pos
	set allres [lindex $val $pos]
	foreach i $allres {
	    set resname [lindex $i 0]
	    regexp {^(.*)[ ]+[(][^\)]*[)]$} $resname {} resname
	    set found 0
	    set fac ""
	    foreach r $validres {
		if { $resname == [lindex $r 0] } {
		    if { [lindex $r 1] == "" } {
		        set fac 1.0
		    } else {
		        set fac [ConvertUnitsToSI $units [lindex $r 1]]
		    }
		    set found 1
		    break
		}
	    }
	    if { !$found } { continue }
	    if { $fac == "" } { return }
	    set type [lindex $i 1]
	    set Results($k,$resname) ""
	    switch $type {
		Beams {
		    foreach j [lindex $i 3] {
		        foreach l [lindex $j 3] {
		            lappend Results($k,$resname) [expr $fac*$l]
		        }
		    }                                     
		}                                     
		Beams1 {                             
		    foreach j [lindex $i 3] {             
		        lappend Results($k,$resname) [lindex $j 0]
		        lappend Results($k,$resname) [lindex $j 1]
		        lappend Results($k,$resname) [lindex $j 2]
		    }
		}
		Triangles {
		    set inum 1
		    foreach j [lindex $i 3] {
		        append Results($k,$resname) " GP$inum"
		        foreach l $j {
		            lappend Results($k,$resname) [expr $fac*$l]
		        }
		        incr inum
		    }
		}
		Triangles1 {
		    foreach j [lindex $i 3] {
		        lappend Results($k,$resname) [lindex $j 0]
		        lappend Results($k,$resname) [lindex $j 1]
		        lappend Results($k,$resname) [lindex $j 2]
		    }
		}
	    }
	}
    }
    .central.s process escape escape escape escape Results AnalysisSel $curr \
	escape escape
    
    
    proc sortres { e1 e2 } {
	regexp {([^,]+),([^,]+)} $e1 {} el1 res1
	regexp {([^,]+),([^,]+)} $e2 {} el2 res2
	if { $el1 < $el2 } { return -1 }
	if { $el1 > $el2 } { return 1 }
	if { $res1 < $res2 } { return -1 }
	if { $res1 > $res2 } { return 1 }
	return 0
    }
    
    set results [lsort -command [namespace code sortres] [array names Results]]
    
    set retval ""
    append retval "NumResults $nres\n"
    append retval "NumLC [llength $alla]\n"
    
    set val [lsearch [.central.s info gendata] Activate_Load_Cases*]
    incr val
    set justone 0
    if { [lindex [.central.s info gendata] $val] == "none" } {
	set justone 1
    }
    set val [lsearch [.central.s info gendata] Combined_load_cases*]
    incr val
    
    set lastLC ""
    foreach i $results {
	regexp {([^,]+),([^,]+)} $i {} LC ip
	if { $LC != $lastLC } {
	    if { $justone } {
		set isstrengths 1
	    } else {
		set val2 [lsearch [lindex [.central.s info gendata] $val] $LC]
		if { $val2 == -1 } {
		    WarnWin "error with loadcases"
		    return
		}
		incr val2
		set isstrengths [lindex [lindex [.central.s info gendata] $val] $val2]
	    }
	    if { $lastLC != "" } {
		append retval "end loadcase\n"
	    }
	    append retval "loadcase $LC $isstrengths\n"
	}
	append retval "$ip $Results($i)\n"
	set lastLC $LC
    }
    append retval "end loadcase\n"
    return $retval
}
