
namespace eval DrawGraph {
    variable c
    variable yvalues
    variable maxx
    variable xlabel
    variable ylabel
    variable title
    variable xfact "" xm "" yfact "" ym "" ymax "" ynummin "" ynummax ""

    proc DrawCurve { cv yvaluesv maxxv xlabelv ylabelv titlev } {
	Init $cv $yvaluesv $maxxv $xlabelv $ylabelv $titlev
	Draw
    }
}

proc DrawGraph::Init { cv yvaluesv maxxv xlabelv ylabelv titlev } {
    variable c $cv
    variable yvalues $yvaluesv
    variable maxx $maxxv
    variable xlabel $xlabelv
    variable ylabel $ylabelv
    variable title $titlev
}


proc DrawGraph::Draw {} {
    variable c
    variable yvalues
    variable maxx
    variable xlabel
    variable ylabel
    variable title
    variable xfact
    variable xm
    variable yfact
    variable ym
    variable ymax
    variable ynummin
    variable ynummax

    $c delete curve
    $c delete axestext
    $c delete titletext
    $c delete zeroline

    set inix 0.0
    set numdivisions [expr [llength $yvalues]-1]

    set ymax [winfo height $c]
    set xmax [winfo width $c]

    set ynummax 0
    set textwidth 0
    for {set i 0 } { $i <= $numdivisions } { incr i } {
	set yval [lindex $yvalues $i]
	if { $yval > $ynummax } {
	    set ynummax $yval
	}
	if { $i == 0 || $yval < $ynummin } {
	    set ynummin $yval
	}
    }
    if { $ynummax == $ynummin } { incr ynummax }

    set inumtics 8
    for {set i 0 } { $i < $inumtics } { incr i } {
	set yvaltext [format "%.4g" [expr $ynummin+$i/double($inumtics-1)*($ynummax-$ynummin)]]
	regsub {e([+-])00} $yvaltext {e\10} yvaltext
	set tt [font measure NormalFont $yvaltext]
	if { $tt > $textwidth } { set textwidth $tt }
    }

    set xm [expr $textwidth+10]
    set textheight [font metrics NormalFont -linespace]
    set ym [expr int($textheight*1.5)+10]

    set fam [font configure NormalFont -family]
    set tsize [expr [font configure NormalFont -size]*2]
    $c create text [expr $xmax/2] 6 -anchor n -justify center \
	    -text $title -font [list $fam $tsize] -tags titletext
    
    $c create line $xm $ym $xm [expr $ymax-$ym] [expr $xmax-$xm] \
	    [expr $ymax-$ym]  -tags axestext

    set inumtics 8
    for {set i 0 } { $i < $inumtics } { incr i } {
	set xvaltext [format "%3.1lf" \
		[expr $inix+$i/double($inumtics-1)*($maxx-$inix)]]
	set xval [expr $xm+$i/double($inumtics-1)*($xmax-2*$xm)]
	set xvalt $xval
	if { $i == 0 } { set xvalt [expr $xvalt+4] }
	$c create line $xval [expr $ymax-$ym+2] $xval [expr $ymax-$ym] \
		-tags axestext
	$c create text $xval [expr $ymax-$ym+2] -anchor n -justify center \
		-text $xvaltext -font NormalFont -tags axestext

	set yvaltext [format "%.4g" [expr $ynummin+$i/double($inumtics-1)*($ynummax-$ynummin)]]
	regsub {e([+-])00} $yvaltext {e\10} yvaltext
	set yval [expr $ymax-$ym-$i/double($inumtics-1)*($ymax-2*$ym)]
	set yvalt $yval
	if { $i == 0 } { set yvalt [expr $yvalt-4] }
	$c create line [expr $xm-2] $yval $xm $yval -tags axestext
	$c create text [expr $xm-3] $yvalt -anchor e -justify right \
		-text $yvaltext -font NormalFont -tags axestext
    }
    $c create text 6 6 -anchor nw -justify left \
	    -text $ylabel -font NormalFont -tags axestext
    set textwidth [font measure NormalFont 0.0]
    $c create text [expr $xmax-$xm+$textwidth] [expr $ymax-$ym] \
	    -anchor nw -justify left \
	    -text $xlabel -font NormalFont -tags axestext

    set xfact [expr ($xmax-2.0*$xm)/double($maxx-$inix)]
    set yfact [expr ($ymax-2.0*$ym)/double($ynummax-$ynummin)]

    set yval [expr $ymax-$ym-(0.0-$ynummin)*$yfact]
    if { $yval > $ym && $yval <= [expr $ymax-$ym] } {
	$c create line $xm $yval [expr $xmax-$xm] $yval -tags zeroline -dash -.-
    }

    set xfactdiv [expr ($xmax-2.0*$xm)/double($numdivisions)]
    set lastyval [expr $ymax-$ym-([lindex $yvalues 0]-$ynummin)*$yfact]
    for {set i 1 } { $i <= $numdivisions } { incr i } {
	set yval [expr $ymax-$ym-([lindex $yvalues $i]-$ynummin)*$yfact]
	$c create line [expr ($i-1)*$xfactdiv+$xm] $lastyval \
		[expr $i*$xfactdiv+$xm] $yval -tags curve -width 3 -fill red
	set lastyval $yval
    }

    $c bind curve <ButtonPress-1> "DrawGraph::DrawGraphCoords %x %y"
}

proc DrawGraph::FindClosestPoint { x y } {
    variable c

    set mindist2 1e20
    foreach i [$c find withtag curve] {
	foreach "ax ay bx by" [$c coords $i] break
	set vx [expr $bx-$ax]
	set vy [expr $by-$ay]
	set alpha [expr $vx*($ax-$x)+$vy*($ay-$y)]
	set landa [expr -1*$alpha/double($vx*$vx+$vy*$vy)]
	if { $landa < 0.0 } { set landa 0.0 }
	if { $landa > 1.0 } { set landa 1.0 }
	set px [expr $ax+$landa*$vx]
	set py [expr $ay+$landa*$vy]

	set dist2 [expr ($px-$x)*($px-$x)+($py-$y)*($py-$y)]
	if { $dist2 < $mindist2 } {
	    set mindist2 $dist2
	    set minpx $px
	    set minpy $py
	}
    }
    return [list $minpx $minpy]
}

proc DrawGraph::DrawGraphCoords { x y } {
    variable c
    variable xlabel
    variable ylabel
    variable xfact
    variable xm
    variable yfact
    variable ym
    variable ymax
    variable ynummin
    variable ynummax
    variable yvalues

    $c delete coords
    $c delete coordpoint

    set ymax [winfo height $c]
    set xmax [winfo width $c]
    
    if { [lindex $yvalues end] < ($ynummax-$ynummin)/2.0 } {
	foreach "{} {} {} ytitle" [$c bbox titletext] break
	set ytitle [expr $ytitle+2]
	set anchor ne
    } else {
	set ytitle [expr $ymax-$ym-5]
	set anchor se
    }

    foreach "xcurve ycurve" [DrawGraph::FindClosestPoint $x $y] break

    $c create oval [expr $xcurve-2] [expr $ycurve-2] [expr $xcurve+2] [expr $ycurve+2] \
	    -tags coordpoint

    set xtext [expr ($xcurve-$xm)/double($xfact)]
    regsub {e([+-])00} $xtext {e\10} xtext
    set ytext [expr ($ymax-$ym-$ycurve)/double($yfact)+$ynummin]
    regsub {e([+-])00} $ytext {e\10} ytext

    $c create text [expr $xmax-6] $ytitle -anchor $anchor -font NormalFont \
	    -text [format "$xlabel: %.4g  $ylabel: %.4g" $xtext $ytext] -tags coords

    $c bind curve <ButtonRelease-1> "$c delete coords coordpoint; $c bind curve <B1-Motion> {}"
    $c bind curve <B1-Motion> "DrawGraph::DrawGraphCoords %x %y"
}
