
package provide mather 1.0

namespace eval math {
    #variable PI [expr {atan2(0,-1)}]
    variable PI 3.1415926535897931
    variable epsilon 1e-11

    namespace export *
}

# non symetric matrixes (stored as lists of lists)
namespace eval math_ns {}

proc math::ConvertLengthsUnitsToSI { system } {
    switch $system {
	"m" { return 1.0 }
	"mm" { return 1e-3 }
	"cm" { return 1e-2 }
    }
    error "Error, using an unknown magnitude for the length units"
}

proc math::ConvertUnitsToSI { system magnitude } {

    switch $system {
	"N-m" { return 1.0 }
	"N-mm" {
	    switch $magnitude {
		N { return 1.0 }
		m { return 1e-3 }
		N/m { return 1e3 }
		Nm { return 1e-3 }
		Pa { return 1e6 }
	    }
	}
	"Kp-cm" {
	    switch $magnitude {
		N { return 9.8 }
		m { return 1e-2 }
		N/m { return 980.0 }
		Nm { return 0.098 }
		Pa { return 98000.0 }
	    }
	}
	"kN-m-MPa" {
	    switch $magnitude {
		N { return 1.0e3 }
		m { return 1.0 }
		N/m { return 1e3 }
		Nm { return 1e3 }
		Pa { return 1e6 }
	    }
	}
    }
    error "Error, using an unknown magnitude for the problem units ($system)"
}

proc math::Normalizevector2D { vec } {
    variable epsilon

    set norm [expr {sqrt([lindex $vec 0]*[lindex $vec 0]+[lindex $vec 1]*[lindex $vec 1])}]

    if { abs($norm) > $epsilon } {
	return [list [expr {[lindex $vec 0]/$norm}] [expr {[lindex $vec 1]/$norm}]]
    }
    return $vec
}

proc math::Normalizevector { vec } {
    variable epsilon

    set norm [expr {sqrt([lindex $vec 0]*[lindex $vec 0]+[lindex $vec 1]*[lindex $vec 1]+\
	[lindex $vec 2]*[lindex $vec 2])}]

    if { abs($norm) > $epsilon } {
	return [list [expr {[lindex $vec 0]/$norm}] [expr {[lindex $vec 1]/$norm}] \
	    [expr {[lindex $vec 2]/$norm}] ]
    }
    return $vec
}

proc math::Angle2D { vec1 vec2 } {
    set mod1 [math::module_2D $vec1]
    set mod2 [math::module_2D $vec2]


    set expr [expr {1.0/$mod1/$mod2*([lindex $vec1 0]*[lindex $vec2 0]+\
		                         [lindex $vec1 1]*[lindex $vec2 1])}]
    if { $expr < -1.0 } { set expr -1.0 } elseif { $expr > 1.0 } { set expr 1.0 }
    set a [expr {acos($expr)}]
    if { [lindex $vec1 0]*[lindex $vec2 1]-[lindex $vec2 0]*[lindex $vec1 1] < 0 } {
	set a [expr {-1.0*$a}]
    }
    return $a
}

proc math::MinAngle2D { p1 p2 p3 } {
    variable PI

    set vec(1) [math::VectorDiff2D $p2 $p1]
    set vec(2) [math::VectorDiff2D $p3 $p2]
    set vec(3) [math::VectorDiff2D $p1 $p3]

    for { set i 1 } { $i <= 3 } { incr i } {
	set mod($i) [math::module_2D $vec($i)]
    }
    set expr [expr {-1.0/$mod(1)/$mod(2)*([lindex $vec(1) 0]*[lindex $vec(2) 0]+\
		                              [lindex $vec(1) 1]*[lindex $vec(2) 1])}]
    if { $expr < -1.0 } { set expr -1.0 } elseif { $expr > 1.0 } { set expr 1.0 }
    set a1 [expr {acos($expr)}]
    if { [lindex $vec(1) 0]*[lindex $vec(2) 1]-[lindex $vec(2) 0]*[lindex $vec(1) 1] < 0 } {
	set a1 [expr {-1.0*$a1}]
    }
    set expr [expr {-1.0/$mod(1)/$mod(3)*([lindex $vec(1) 0]*[lindex $vec(3) 0]+\
		                              [lindex $vec(1) 1]*[lindex $vec(3) 1])}]
    if { $expr < -1.0 } { set expr -1.0 } elseif { $expr > 1.0 } { set expr 1.0 }
    set a2 [expr {acos($expr)}]
    if { -1*([lindex $vec(1) 0]*[lindex $vec(3) 1]-[lindex $vec(3) 0]*[lindex $vec(1) 1]) < 0 } {
	set a2 [expr {-1.0*$a2}]
    }
    set a3 [expr {$PI-$a1-$a2}]

    if { $a1 < $a2 && $a1 < $a3 } { return $a1 }
    if { $a2 < $a3 } { return $a2 }
    return $a3
}

proc math::VectorProd { vec1 vec2 } {

    return [list [expr {[lindex $vec1 1]*[lindex $vec2 2]-[lindex $vec2 1]*[lindex $vec1 2]}] \
	[expr {[lindex $vec1 2]*[lindex $vec2 0]-[lindex $vec2 2]*[lindex $vec1 0]}] \
	[expr {[lindex $vec1 0]*[lindex $vec2 1]-[lindex $vec2 0]*[lindex $vec1 1]}]]
}

proc math::DotProd { vec1 vec2 } {
    return [expr {[lindex $vec1 0]*[lindex $vec2 0]+ \
	    [lindex $vec1 1]*[lindex $vec2 1]+ \
	    [lindex $vec1 2]*[lindex $vec2 2]}]
}

# returns vec1+vec2
proc math::VectorAdd2D { args } {

    set v [lindex $args 0]
    foreach i [lrange $args 1 end] {
	lset v 0 [expr {[lindex $v 0]+[lindex $i 0]}]
	lset v 1 [expr {[lindex $v 1]+[lindex $i 1]}]
    }
    return $v
}
# returns vec1+vec2
proc math::VectorAdd { vec1 vec2 } {
    return [list [expr {[lindex $vec1 0]+[lindex $vec2 0]}] \
	[expr {[lindex $vec1 1]+[lindex $vec2 1]}] \
	[expr {[lindex $vec1 2]+[lindex $vec2 2]}]]
}

# returns vec1-vec2
proc math::VectorDiff2D { vec1 vec2 } {
    return [list [expr {[lindex $vec1 0]-[lindex $vec2 0]}] \
	[expr {[lindex $vec1 1]-[lindex $vec2 1]}]]
}

# returns vec1-vec2
proc math::VectorDiff { vec1 vec2 } {
    return [list [expr {[lindex $vec1 0]-[lindex $vec2 0]}] \
	[expr {[lindex $vec1 1]-[lindex $vec2 1]}] \
	[expr {[lindex $vec1 2]-[lindex $vec2 2]}]]
}

proc math::module_2D { vec1 } {
    return [expr {sqrt([lindex $vec1 0]*[lindex $vec1 0]+\
	[lindex $vec1 1]*[lindex $vec1 1])}]
}

proc math::module { vec1 } {
    return [expr {sqrt([lindex $vec1 0]*[lindex $vec1 0]+\
	[lindex $vec1 1]*[lindex $vec1 1]+\
	[lindex $vec1 2]*[lindex $vec1 2])}]
}

proc math::module2_2D { vec1 } {
    return [expr {[lindex $vec1 0]*[lindex $vec1 0]+\
	[lindex $vec1 1]*[lindex $vec1 1]}]
}

proc math::dist_2D { vec1 vec2 } {
    return [module_2D [math::VectorDiff2D $vec1 $vec2]]
}

proc math::dist_3D { vec1 vec2 } {
    return [module [math::VectorDiff $vec1 $vec2]]
}

proc math::AreEqual { vec1 vec2 eps } {
    return [expr {[module [math::VectorDiff $vec1 $vec2]] < $eps}]
}

proc math::AreOpposite { vec1 vec2 eps } {
    return [expr {[module [math::VectorAdd $vec1 $vec2]] < $eps}]
}

proc math::module2 { vec1 } {
    return [expr {[lindex $vec1 0]*[lindex $vec1 0]+\
	[lindex $vec1 1]*[lindex $vec1 1]+\
	[lindex $vec1 2]*[lindex $vec1 2]}]
}

proc math::ScalarDotVector2D { sc vec } {
    return [list [expr {$sc*[lindex $vec 0]}] [expr {$sc*[lindex $vec 1]}]]
}

proc math::ScalarDotVector { sc vec } {
    set len [llength $vec]
    set retvec ""
    for { set i 0 } { $i < $len } { incr i } {
	lappend retvec [expr {$sc*[lindex $vec $i]}]
    }
    return $retvec
}

# from 1 to 3
proc math::val { mat row col } {
    return [lindex $mat [expr {$col-1+3*($row-1)}]]
}

# from 1 to 2
proc math::set2 { matname row col newvalue } {
    upvar 1 $matname mat
    lset mat [expr {$col-1+2*($row-1)}] $newvalue
}

# from 1 to 3
proc math::set3 { matname row col newvalue } {
    upvar 1 $matname mat
    lset mat [expr {$col-1+3*($row-1)}] $newvalue
}

# from 1 to 2
proc math::get2 { matname row col } {
    upvar 1 $matname mat
    return [lindex $mat [expr {$col-1+2*($row-1)}]]
}

# from 1 to 3
proc math::get3 { matname row col } {
    upvar 1 $matname mat
    return [lindex $mat [expr {$col-1+3*($row-1)}]]
}

proc math::RotMat2OpenGLmat { mat } {
    
    return [list \
	[lindex $mat 0] [lindex $mat 3] [lindex $mat 6] 0 \
	[lindex $mat 1] [lindex $mat 4] [lindex $mat 7] 0 \
	[lindex $mat 2] [lindex $mat 5] [lindex $mat 8] 0 \
	0                0               0       1.0]
}

# axes must be: x,y or z (it is a column)
proc math::SetAxeInMatrix { mat axe values } {
    set axe [lsearch "x y z" $axe]
    for { set i 0 } { $i < 3 } { incr i } {
	lset mat [expr {$axe+3*$i}] [lindex $values $i]
    }
    return $mat
}

# axes must be: x,y or z (it is a column)
proc math::GetAxeInMatrix { mat axe } {
    set axe [lsearch "x y z" $axe]
    set values ""
    for { set i 0 } { $i < 3 } { incr i } {
	lappend values [lindex $mat [expr {$axe+3*$i}]]
    }
    return $values
}


proc math::BeamTransfMatrix { elm nodesname } {
    variable epsilon

    upvar $nodesname nodes

    set iprima [Normalizevector [list \
	[expr {[lindex $nodes([lindex $elm 1]) 0]-[lindex $nodes([lindex $elm 0]) 0]}] \
	[expr {[lindex $nodes([lindex $elm 1]) 1]-[lindex $nodes([lindex $elm 0]) 1]}] \
	[expr {[lindex $nodes([lindex $elm 1]) 2]-[lindex $nodes([lindex $elm 0]) 2]}]]]

    if { abs([lindex $iprima 0]) > $epsilon || abs([lindex $iprima 1]) } {
	if { abs([lindex $iprima 0]) > $epsilon } {
	    set jprima [list [expr {-1*[lindex $iprima 1]/double([lindex $iprima 0])}] 1.0 0.0]
	} else {
	    set jprima [list 1.0 0.0 0.0]
	}
	set jprima [Normalizevector $jprima]
	set kprima [VectorProd $iprima $jprima]
	if { abs([lindex $kprima 2]) < $epsilon } {
	    set jprima [ScalarDotVector -1.0 $jprima]
	    set kprima [VectorProd $iprima $jprima]
	}
    } else {
	set jprima [list 1.0 0.0 0.0]
	set kprima [VectorProd $iprima $jprima]
    }
    return [list \
	[lindex $iprima 0] [lindex $jprima 0] [lindex $kprima 0] \
	[lindex $iprima 1] [lindex $jprima 1] [lindex $kprima 1] \
	[lindex $iprima 2] [lindex $jprima 2] [lindex $kprima 2]]
}

proc math::CorrectBeamTransfMatrix { elm nodesname mat } {
    variable epsilon

    upvar $nodesname nodes

    set xaxe [Normalizevector [GetAxeInMatrix $mat x]]
    
    set iprima [Normalizevector [list \
	[expr {[lindex $nodes([lindex $elm 1]) 0]-[lindex $nodes([lindex $elm 0]) 0]}] \
	[expr {[lindex $nodes([lindex $elm 1]) 1]-[lindex $nodes([lindex $elm 0]) 1]}] \
	[expr {[lindex $nodes([lindex $elm 1]) 2]-[lindex $nodes([lindex $elm 0]) 2]}]]]

    set dotprod [DotProd $xaxe $iprima]
    if { abs($dotprod)>=1.0-$epsilon } { return $mat }

    set changeZaxe 1
    if { [val $mat 3 3] > $epsilon } { set changeZaxe 0 }
    if { $dotprod < -$epsilon } {
	set mat [SetAxeInMatrix $mat x [ScalarDotVector -1.0 $xaxe]]
	if { $changeZaxe } {
	    set mat [SetAxeInMatrix $mat z [ScalarDotVector -1.0 [GetAxeInMatrix $mat z]]]
	} else {
	    set mat [SetAxeInMatrix $mat y [ScalarDotVector -1.0 [GetAxeInMatrix $mat y]]]
	}
	set dotprod [expr {$dotprod*-1.0}]
    }
    if { abs($dotprod)>=1.0-$epsilon } { return $mat }
    if { $dotprod <= $epsilon } { return $mat } ;# this is an error: x orthogonal to beam

    set jprima [Normalizevector [GetAxeInMatrix $mat y]]
    set kprima [Normalizevector [VectorProd $iprima $jprima]]
    set jprima [VectorProd $kprima $iprima]

    set mat [SetAxeInMatrix $mat x $iprima]
    set mat [SetAxeInMatrix $mat y $jprima]
    set mat [SetAxeInMatrix $mat z $kprima]

    return $mat
}

proc math::ShellTransfMatrix  { elm nodesname } {
    variable epsilon

    upvar $nodesname nodes

    set kprima [Normalizevector [VectorProd \
	[list \
	    [expr {[lindex $nodes([lindex $elm 1]) 0]-[lindex $nodes([lindex $elm 0]) 0]}] \
	    [expr {[lindex $nodes([lindex $elm 1]) 1]-[lindex $nodes([lindex $elm 0]) 1]}] \
	    [expr {[lindex $nodes([lindex $elm 1]) 2]-[lindex $nodes([lindex $elm 0]) 2]}]] \
	[list \
	    [expr {[lindex $nodes([lindex $elm 2]) 0]-[lindex $nodes([lindex $elm 0]) 0]}] \
	    [expr {[lindex $nodes([lindex $elm 2]) 1]-[lindex $nodes([lindex $elm 0]) 1]}] \
	    [expr {[lindex $nodes([lindex $elm 2]) 2]-[lindex $nodes([lindex $elm 0]) 2]}]] \
       ]]
    
    if { abs([lindex $kprima 0]) > $epsilon || abs([lindex $kprima 2]) > $epsilon } {
	set iprima [Normalizevector [VectorProd [list 0.0 1.0 0.0] $kprima]]
    } else {
	set iprima [Normalizevector [VectorProd [list 0.0 0.0 1.0] $kprima]]
    }
    set jprima [VectorProd $kprima $iprima]

    return [list \
	[lindex $iprima 0] [lindex $jprima 0] [lindex $kprima 0] \
	[lindex $iprima 1] [lindex $jprima 1] [lindex $kprima 1] \
	[lindex $iprima 2] [lindex $jprima 2] [lindex $kprima 2]]
}

# it is suposed a rectangular matrix
proc math::MatriuTranspose { mat } {

    set n [expr {int(sqrt([llength $mat]))}]

    set retmat ""
    for { set i 0 } { $i < $n } { incr i } {
	for { set j 0 } { $j < $n } { incr j } {
	    lappend retmat [lindex $mat [expr {$j*$n+$i}]]
	}
    }
    return $retmat
}

proc math::MatriuPerVec { mat vec } {

    set n [llength $vec]
    set ret ""
    for { set i 0 } { $i < $n } { incr i } {
	set val 0.0
	for { set j 0 } { $j < $n } { incr j } {
	    set val [expr {$val+[lindex $mat [expr {$j+$i*$n}]]*[lindex $vec $j]}]
	}
	lappend ret $val
    }
    return $ret
}

proc math::Matriu3PerVec2 { mat vec } {

    set ret ""
    for { set i 0 } { $i < 2 } { incr i } {
	set val 0.0
	for { set j 0 } { $j < 2 } { incr j } {
	    set val [expr {$val+[lindex $mat [expr {$j+$i*3}]]*[lindex $vec $j]}]
	}
	lappend ret [expr {($val+[lindex $mat [expr {$j+$i*3}]])*[lindex $mat 8]}]
    }
    return $ret
}

proc math::Matriu4PerVec3 { mat vec } {

    set ret ""
    for { set i 0 } { $i < 3 } { incr i } {
	set val 0.0
	for { set j 0 } { $j < 3 } { incr j } {
	    set val [expr {$val+[lindex $mat [expr {$j+$i*4}]]*[lindex $vec $j]}]
	}
	lappend ret [expr {($val+[lindex $mat [expr {$j+$i*4}]])*[lindex $mat 15]}]
    }
    return $ret
}

proc math::MatriuPerMatriu { mat1 mat2 } {
    set n [expr {int(sqrt([llength $mat1]))}]

    set mat ""
    for { set i 0 } { $i < $n } { incr i } {
	for { set j 0 } { $j < $n } { incr j } {
	    set val 0.0
	    for { set k 0 } { $k < $n } { incr k } {
		set val [expr {$val+[lindex $mat1 [expr {$i*$n+$k}]]*[lindex $mat2 [expr {$k*$n+$j}]]}]
	    }
	    lappend mat $val
	}
    }
    return $mat
}

proc math::MatriuTPerVec { mat vec } {

    set n [llength $vec]
    set ret ""
    for { set i 0 } { $i < $n } { incr i } {
	set val 0.0
	for { set j 0 } { $j < $n } { incr j } {
	    set val [expr {$val+[lindex $mat [expr {$i+$j*$n}]]*[lindex $vec $j]}]
	}
	lappend ret $val
    }
    return $ret
}

proc math::GiveTriangleProjectedNatCoord { elm nodesname trielmsname pnt } {
    upvar $nodesname nodes
    upvar $trielmsname trielms

    set mat [ShellTransfMatrix [lrange $trielms($elm) 0 2] nodes]
    set tmppnt [MatriuTPerVec $mat $pnt]
    for { set i 0 } { $i < 3 } { incr i } {
	set tmpnodes($i) [MatriuTPerVec $mat $nodes([lindex $trielms($elm) $i])]
    }
    
    set retheight [expr {abs([lindex $tmppnt 2]-[lindex $tmpnodes(0) 2])}]
      
    set ax [expr {[lindex $tmpnodes(1) 0]-[lindex $tmpnodes(0) 0]}]
    set ay [expr {[lindex $tmpnodes(1) 1]-[lindex $tmpnodes(0) 1]}]
    
    set bx [expr {[lindex $tmpnodes(2) 0]-[lindex $tmpnodes(0) 0]}]
    set by [expr {[lindex $tmpnodes(2) 1]-[lindex $tmpnodes(0) 1]}]
    
    set cx [expr {[lindex $tmppnt 0]-[lindex $tmpnodes(0) 0]}]
    set cy [expr {[lindex $tmppnt 1]-[lindex $tmpnodes(0) 1]}]
    
    if { abs($ax) < $math::epsilon } {
	set N2 [expr {$cx/double($bx)}]
	set N1 [expr {($cy-$by*$N2)/double($ay)}]
    } else {
	set N2 [expr {($cy-$cx*$ay/double($ax))/double($by-$bx*$ay/double($ax))}]
	set N1 [expr {($cx-$bx*$N2)/double($ax)}]
    }
    return [list [expr {1.0-$N1-$N2}] $N1 $N2 $retheight]
}

proc math::CalcValueInTrianglePoint { elm nodesname trielmsname pnt gaussvalues } {
    variable LUmatTri6nodes

    upvar $nodesname nodes
    upvar $trielmsname trielms

    if { ![info exists LUmatTri6nodes] } {
	set LUmatTri6nodes [LU [list 0.5 0.5 0.0 \
		                     0.0 0.5 0.5 \
		                     0.5 0.0 0.5]]
    }
    foreach "N(0) N(1) N(2) H" [GiveTriangleProjectedNatCoord $elm nodes trielms $pnt] break
    for { set i 0 } { $i < 2 } { incr i } {
	if { $N($i) < -.01 || $N($i) > 1.01 } {
	    error "error N($i)=$N($i) 0 <= $i <= 2"
	}
    }
    set nodalvalues [LULinSolve $LUmatTri6nodes $gaussvalues]
    set val 0
    for { set i 0 } { $i < 3 } { incr i } {
	set val [expr {$val+$N($i)*[lindex $nodalvalues $i]}]
    }
    return $val
}

proc math::CircleCenterAndNormal { p1 p2 p3 } {

    set normal [Normalizevector [VectorProd [VectorDiff $p2 $p1] [VectorDiff $p3 $p1]]]
    set xaxe [Normalizevector [VectorProd [VectorDiff $p2 $p1] $normal]]
    set x0 [ScalarDotVector .5 [VectorAdd $p1 $p2]]
    set T [CalcMatGivenXZAxes $xaxe $normal]
    set Tt [MatriuTranspose $T]

    set x0_L [MatriuPerVec $Tt $x0]
    foreach i [list 1 2 3] {
	set p${i}_L [MatriuPerVec $Tt [set p$i]]
    }
    set x2 [ScalarDotVector .5 [VectorAdd $p1 $p3]]
    set x2_L [MatriuPerVec $Tt $x2]
    set n2 [Normalizevector [VectorProd [VectorDiff $p3 $p1] $normal]]
    set n2_L [MatriuPerVec $Tt $n2]
    set landa [expr {([lindex $x0_L 1]-[lindex $x2_L 1])/double([lindex $n2_L 1])}]
    set center_L [VectorAdd $x2_L [ScalarDotVector $landa $n2_L]]
    set center [MatriuPerVec $T $center_L]
    return [list $center $normal]
}

#math::CircleCenterAndNormal "1 2 3" "23.4 45.6 0" "-3 4 -89"

proc math::ElementCenter { elm nodesname } {
    upvar $nodesname nodes

    set center [list 0.0 0.0 0.0]
    set len [llength $elm]
    for { set i 0 } { $i < $len } { incr i } {
	lset center 0 [expr {[lindex $center 0]+[lindex $nodes([lindex $elm $i]) 0]}]
	lset center 1 [expr {[lindex $center 1]+[lindex $nodes([lindex $elm $i]) 1]}]
	lset center 2 [expr {[lindex $center 2]+[lindex $nodes([lindex $elm $i]) 2]}]
    }
    lset center 0 [expr {[lindex $center 0]/double($len)}]
    lset center 1 [expr {[lindex $center 1]/double($len)}]
    lset center 2 [expr {[lindex $center 2]/double($len)}]
    return $center
}

proc math::LU { mat } {
    # Descomposicin de Crout M=L*U de una matriz cuadrada
    # L matriz triangular inferior, U matriz triangular superior (con unos en la diagonal)
    # se retornan L y U guardadas en una matriz nica 
    # (la diagonal de U se sobreescribe, pero se sabe que es: 1.0,1.0,...,1.0)

    set n [expr {int(sqrt([llength $mat]))}]
    set LUmat [lrepeat [expr {$n*$n}] 0.0]

    proc idx { r c n } { return [expr {($r-1)*$n+$c-1}] }

    set LU11 [expr {double([lindex $mat [idx 1 1 $n]])}]
    lset LUmat [idx 1 1 $n] $LU11 ;# L(1,1)=m(1,1); U(1,1)=1.0;
    for { set k 2 } { $k <= $n } { incr k } {
	lset LUmat [idx 1 $k $n] [expr {[lindex $mat [idx 1 $k $n]]/$LU11}]

	for { set i 2 } { $i < $k } { incr i } {
	    set suma 0.0
	    for { set j 1 } { $j < $i } { incr j } {
		set suma [expr {$suma+[lindex $LUmat [idx $i $j $n]]*[lindex $LUmat [idx $j $k $n]]}]
	    }
	    set LUii [expr {double([lindex $LUmat [idx $i $i $n]])}]
	    lset LUmat [idx $i $k $n] [expr {([lindex $mat [idx $i $k $n]]-$suma)/$LUii}]
	}
	lset LUmat [idx $k 1 $n] [lindex $mat [idx $k 1 $n]]

	for { set i 2 } { $i < $k } { incr i } {
	    set suma 0.0
	    for { set j 1 } { $j < $i } { incr j } {
		set suma [expr {$suma+[lindex $LUmat [idx $j $i $n]]*[lindex $LUmat [idx $k $j $n]]}]
	    }
	    lset LUmat [idx $k $i $n] [expr {[lindex $mat [idx $k $i $n]]-$suma}]
	}

	set suma 0.0
	for { set j 1 } { $j < $k } { incr j } {
	    set suma [expr {$suma+[lindex $LUmat [idx $k $j $n]]*[lindex $LUmat [idx $j $k $n]]}]
	}
	lset LUmat [idx $k $k $n] [expr {[lindex $mat [idx $k $k $n]]-$suma}]
    }
    return $LUmat
}

proc math::LULinSolve { LUmat b } {
    # solve the linear system (LU)*x=b     (LU= crout decomposition of M) 
    # returns x. on error, returns ""

    set n [expr {int(sqrt([llength $LUmat]))}]
    proc idx { r c n } { return [expr {($r-1)*$n+$c-1}] }

    set LU11 [expr {double([lindex $LUmat [idx 1 1 $n]])}]

    set eps 1e-15
    if { abs($LU11) < $eps } { error "error: cannot solve" }

    lset b 0 [expr {[lindex $b 0]/$LU11}]
    for { set i 2 } { $i <= $n } { incr i } {
	set LUii [expr {double([lindex $LUmat [idx $i $i $n]])}]
	if { abs($LUii) < $eps } { error "error: cannot solve" }
	set suma 0.0
	for { set j 1 } { $j < $i } { incr j } {
	    set suma [expr {$suma+[lindex $LUmat [idx $i $j $n]]*[lindex $b [expr {$j-1}]]}]
	}
	lset b [expr {$i-1}] [expr {([lindex $b [expr {$i-1}]]-$suma)/$LUii}]
    }
    # sustitucin adelante para resolver U*x=y ==>x
    for { set i [expr {$n-1}] } { $i >= 1 } { incr i -1 } {
	set suma 0.0
	for { set j [expr {$i+1}] } { $j <= $n } { incr j } {
	    set suma [expr {$suma+[lindex $LUmat [idx $i $j $n]]*[lindex $b [expr {$j-1}]]}]
	}
	lset b [expr {$i-1}] [expr {[lindex $b [expr {$i-1}]]-$suma}]
    }
    return $b
}


proc K {a b} {set a}
# if {[string equal [info command ::lset] {}]} {
#     proc ::lset {listName index val} {
#         upvar $listName list
#         set list [lreplace [K $list [set list {}]] $index $index $val]
#     }
# }

if { [info command lrepeat] == "" } {
    proc lrepeat { count element } {
	set retval ""
	for { set i 0 } { $i < $count } { incr i } {
	    lappend retval $element
	}
	return $retval
    }
}

proc math::Identity { dim } {

    set mat ""
    for { set i 0 } { $i < $dim } { incr i } {
	for { set j 0 } { $j < $dim } { incr j } {
	    if { $i == $j } {
		lappend mat 1.0
	    } else { lappend mat 0.0 }
	}
    }
    return $mat
}

proc math::nullMatrix { dim } {
    return [lrepeat [expr {$dim*$dim}] 0.0]
}

proc math::ChangeVecDimension { vec newdimension { newval 0.0 } } {
    if { [llength $vec] >= $newdimension } {
	return [lrange $vec 0 [expr {$newdimension-1}]]
    }
    eval lappend vec [lrepeat [expr {$newdimension-[llength $vec]}] $newval]
    return $vec
}

# matrix is 4x4
proc math::TranslateMatrix { mat vector } { 
    lset mat 3 [lindex $vector 0]
    lset mat 7 [lindex $vector 1]
    lset mat 11 [lindex $vector 2]
    return $mat
}

# matrix is 4x4
proc math::RotateMatrix { mat axe angle } {
    set c [expr {cos($angle)}]
    set s [expr {sin($angle)}]

    lset mat 0 [expr {[lindex $axe 0]*[lindex $axe 0]+$c*(1.0-[lindex $axe 0]*[lindex $axe 0])}]
    lset mat 5 [expr {[lindex $axe 1]*[lindex $axe 1]+$c*(1.0-[lindex $axe 1]*[lindex $axe 1])}]
    lset mat 10 [expr {[lindex $axe 2]*[lindex $axe 2]+$c*(1.0-[lindex $axe 2]*[lindex $axe 2])}]

    lset mat 1 [expr {[lindex $axe 0]*[lindex $axe 1]*(1.0-$c)-[lindex $axe 2]*$s}]
    lset mat 2 [expr {[lindex $axe 2]*[lindex $axe 0]*(1.0-$c)+[lindex $axe 1]*$s}]

    lset mat 4 [expr {[lindex $axe 0]*[lindex $axe 1]*(1.0-$c)+[lindex $axe 2]*$s}]
    lset mat 6 [expr {[lindex $axe 1]*[lindex $axe 2]*(1.0-$c)-[lindex $axe 0]*$s}]

    lset mat 8 [expr {[lindex $axe 2]*[lindex $axe 0]*(1.0-$c)-[lindex $axe 1]*$s}]
    lset mat 9 [expr {[lindex $axe 1]*[lindex $axe 2]*(1.0-$c)+[lindex $axe 0]*$s}]
    return $mat
}

# matrix is 3x3
proc math::TranslateMatrix2D { vector } {
    set mat [Identity 3]
    lset mat 2 [lindex $vector 0]
    lset mat 5 [lindex $vector 1]
    lset mat 8 1.
    return $mat
}

# matrix is 3x3
proc math::ScaleMatrix2D { scalex scaley } {
    set mat [Identity 3]
    lset mat 0 $scalex
    lset mat 4 $scaley
    return $mat
}

# matrix is 3x3
proc math::RotateMatrix2D { center angle } {
    set c [expr {cos($angle)}]
    set s [expr {sin($angle)}]

    set mat [Identity 3]
    lset mat 0 $c
    lset mat 1 [expr {-1*$s}]
    lset mat 2 [expr {-1*$c*[lindex $center 0]+$s*[lindex $center 1]+[lindex $center 0]}]
    lset mat 3 $s
    lset mat 4 $c
    lset mat 5 [expr {-1*$s*[lindex $center 0]-$c*[lindex $center 1]+[lindex $center 1]}]
    lset mat 6 0
    lset mat 7 0
    lset mat 8 1
    return $mat
}


proc math::RotationMatrix { axe1 axe2 angle } {

    set axe [Normalizevector [VectorDiff $axe2 $axe1]]

    set mat1 [TranslateMatrix [Identity 4] [ScalarDotVector -1.0 $axe1]]
    set mat2 [RotateMatrix [Identity 4] $axe $angle]
    set mat [MatriuPerMatriu $mat2 $mat1]
    set mat3 [TranslateMatrix [Identity 4] $axe1]
    return [MatriuPerMatriu $mat3 $mat]
}

proc math::TranslationMatrix { p1 p2 } {

    set vec [VectorDiff $p2 $p1]
    return [TranslateMatrix [Identity 4] $vec]
}


proc math::determinant { mat } {

    set n [expr {int(sqrt([llength $mat]))}]

    switch $n {
	1 { return [lindex $mat 0] }
	2 { return [expr {[lindex $mat 0]*[lindex $mat 3]-[lindex $mat 1]*[lindex $mat 2]}] }
	3 { return [expr {[lindex $mat 0]*[lindex $mat 4]*[lindex $mat 8]-\
		              [lindex $mat 7]*[lindex $mat 5]+\
		              [lindex $mat 3]*[lindex $mat 7]*[lindex $mat 2]-\
		              [lindex $mat 1]*[lindex $mat 8]+\
		              [lindex $mat 6]*[lindex $mat 1]*[lindex $mat 5]-\
		              [lindex $mat 4]*[lindex $mat 2]}] }
	default {
	    error "not implemented yet a determinant bigger than 3"
	}
    }
}

# calculates arbitrary matrix with axe z=normal
# normal must be of norma 1.0
proc math::CalcMatGivenOneAxe { normal } {
    
    set Uy [list 0.0 1.0 0.0]
    set Uz [list 0.0 0.0 1.0]
    if { abs([lindex $normal 0]) < 1.0/64.0 && abs([lindex $normal 1]) < 1.0/64.0 } {
	set Ax [Normalizevector [VectorProd $Uy $normal]]
    } else {
	set Ax [Normalizevector [VectorProd $Uz $normal]]
    }
    set Ay [VectorProd $normal $Ax]

    set mat [Identity 3]
    set mat [SetAxeInMatrix $mat x $Ax]
    set mat [SetAxeInMatrix $mat y $Ay]
    set mat [SetAxeInMatrix $mat z $normal]
    return $mat
}

# similar to func before but it tries to maintain x axe
# normal must be of norma 1.0
proc math::CalcMatGivenZAxe { normal } {
    variable epsilon

    set Uy [list 0.0 1.0 0.0]
    set Uz [list 0.0 0.0 1.0]
    if { abs([lindex $normal 0]) > $epsilon || abs([lindex $normal 2]) > $epsilon } {
	set Ax [Normalizevector [VectorProd $Uy $normal]]
    } else {
	set Ax [Normalizevector [VectorProd $Uz $normal]]
    }
    set Ay [VectorProd $normal $Ax]

    set mat [Identity 3]
    set mat [SetAxeInMatrix $mat x $Ax]
    set mat [SetAxeInMatrix $mat y $Ay]
    set mat [SetAxeInMatrix $mat z $normal]
    return $mat
}


# x and z must be more or less orthogonal. x is mantained
# x and z must be of norma 1.0
proc math::CalcMatGivenXZAxes { xaxe zaxe } {

    set yaxe [Normalizevector [VectorProd $zaxe $xaxe]]
    set zaxe [VectorProd $xaxe $yaxe]

    set mat [Identity 3]
    set mat [SetAxeInMatrix $mat x $xaxe]
    set mat [SetAxeInMatrix $mat y $yaxe]
    set mat [SetAxeInMatrix $mat z $zaxe]
    return $mat
}

proc math::GiveMatrixFromEuler { e } {
    set cosA [expr {cos([lindex $e 0])}]
    set sinA [expr {sin([lindex $e 0])}]
    set cosB [expr {cos([lindex $e 1])}]
    set sinB [expr {sin([lindex $e 1])}]
    set cosC [expr {cos([lindex $e 2])}]
    set sinC [expr {sin([lindex $e 2])}]

    proc idx { r c n } { return [expr {($r-1)*$n+$c-1}] }
    
    set mat [lrepeat 9 0.0] 
    lset mat [idx 1 1 3] [expr {$cosC*$cosA-1*$cosB*$sinA*$sinC}]
    lset mat [idx 2 1 3] [expr {-1*$sinC*$cosA-$cosB*$sinA*$cosC}]
    lset mat [idx 3 1 3] [expr {$sinB*$sinA}]

    lset mat [idx 1 2 3] [expr {$cosC*$sinA+$cosB*$cosA*$sinC}]
    lset mat [idx 2 2 3] [expr {-1*$sinC*$sinA+$cosB*$cosA*$cosC}]
    lset mat [idx 3 2 3] [expr {-1*$sinB*$cosA}]

    lset mat [idx 1 3 3] [expr {$sinC*$sinB}]
    lset mat [idx 2 3 3] [expr {$cosC*$sinB}]
    lset mat [idx 3 3 3] [expr {$cosB}]

    return $mat
}


proc math::GiveEulerAnglesFromMatrix { mat } {
    variable epsilon
    variable PI

    proc idx { r c n } { return [expr {($r-1)*$n+$c-1}] }
    
    set mat33 [lindex $mat [idx 3 3 3]]
    if { $mat33 < 1.0-$epsilon && $mat33 > -1.0+$epsilon } {
	set senb [expr {sqrt(1.0-$mat33*$mat33)}]
	set e2 [expr {acos($mat33)}]
	set e3 [expr {acos([lindex $mat [idx 2 3 3]]/$senb)}]
	if { [lindex $mat [idx 1 3 3]]/$senb < 0.0 } {
	    set e3 [expr {2.0*$PI-$e3}]
	}
	set e1 [expr {acos(-1.0*[lindex $mat [idx 3 2 3]]/$senb)}]
	if { [lindex $mat [idx 3 1 3]]/$senb < 0.0 } {
	    set e1 [expr {2.0*$PI-$e1}]
	}
    } else {
	set e2 [expr {acos([lindex $mat [idx 3 3 3]])}]
	set e1 0.0
	set e3 [expr {acos([lindex $mat [idx 1 1 3]])}]
	if { -1.0*[lindex $mat [idx 2 1 3]] < 0.0 } {
	    set e3 [expr {2.0*$PI-$e3}]
	}
    }
    return [list $e1 $e2 $e3]
}

proc math::give6x6transfmatrix_from_3x3 { T } {

    set T6x6 [Identity 6]
    for { set i 0 } { $i < 3 } { incr i } {
	for { set j 0 } { $j < 3 } { incr j } {
	    lset T6x6 [expr {$i*6+$j}] [lindex $T [expr {$i*3+$j}]]
	}
    }
    for { set i 0 } { $i < 3 } { incr i } {
	lset T6x6 [expr {$i*6+18+3}] [expr {-1*[lindex $T [expr {$i*3+1}]]}]
	lset T6x6 [expr {$i*6+18+4}] [lindex $T [expr {$i*3+0}]]
	lset T6x6 [expr {$i*6+18+5}] [lindex $T [expr {$i*3+2}]]
    }
    return $T6x6
}

proc math_ns::create { nrow ncol } {
    return [lrepeat $nrow [lrepeat $ncol 0.0]]
}

proc math_ns::nullMatrix { nrow ncol } {
    return [lrepeat $nrow [lrepeat $ncol 0.0]]
}

proc math_ns::change_dimension { matname nrow ncol } {
    upvar 1 $matname mat

    set nrow1 [llength $mat]
    set ncol1 [llength [lindex $mat 0]]

    set newmat [create $nrow $ncol]
    for { set i 0 } { $i < $nrow1 && $i < $nrow } { incr i } {
	for { set j 0 } { $j < $ncol1 && $j < $ncol } { incr j } {
	    lset newmat $i $j [lindex $mat $i $j]
	}
    }
    set mat $newmat
}

proc math_ns::setv { matname row col val } {
    upvar 1 $matname mat
    lset mat [expr {$row-1}] [expr {$col-1}] $val
}

proc math_ns::getv { matname row col } {
    upvar 1 $matname mat
    return [lindex $mat [expr {$row-1}] [expr {$col-1}]]
}

proc math_ns::setrow { matname row vals } {
    upvar 1 $matname mat
    set r [expr {$row-1}]
    set mat [lreplace $mat $r $r $vals]
}

proc math_ns::copy_from_sym { mat } {
    set n [expr {int(sqrt([llength $mat]))}]

    set retmat [create $n $n]
    for { set i 0 } { $i < $n } { incr i } {
	for { set j 0 } { $j < $n } { incr j } {
	    lset retmat $i $j [lindex $mat [expr {$i*$n+$j}]]
	}
    }
    return $retmat
}

proc math_ns::MatriuPerMatriu { mat1 mat2 } {
    set nrow1 [llength $mat1]
    set ncol1 [llength [lindex $mat1 0]]

    set nrow2 [llength $mat2]
    set ncol2 [llength [lindex $mat2 0]]

    if { $ncol1 != $nrow2 } {
	error "error in math_ns::MatriuPerMatriu. ncol1!=nrow2 ($ncol1!=$ncol2)"
    }
    set mat [create $nrow1 $ncol2]
    for { set i 0 } { $i < $nrow1 } { incr i } {
	for { set j 0 } { $j < $ncol2 } { incr j } {
	    set val 0.0
	    for { set k 0 } { $k < $ncol1 } { incr k } {
		set val [expr {$val+[lindex $mat1 $i $k]*[lindex $mat2 $k $j]}]
	    }
	    lset mat $i $j $val
	}
    }
    return $mat
}

proc math_ns::MatriuAddMatriu { mat1 mat2 } {
    set nrow1 [llength $mat1]
    set ncol1 [llength [lindex $mat1 0]]

    set nrow2 [llength $mat2]
    set ncol2 [llength [lindex $mat2 0]]

    if { $nrow1 != $nrow2 || $ncol1 != $ncol2 } {
	error "error in math_ns::MatriuAddMatriu"
    }
    set mat [create $nrow1 $ncol1]
    for { set i 0 } { $i < $nrow1 } { incr i } {
	for { set j 0 } { $j < $ncol2 } { incr j } {
	    lset mat $i $j [expr {[lindex $mat1 $i $j]+[lindex $mat2 $i $j]}]
	}
    }
    return $mat
}

proc math_ns::MatriuTranspPerMatriu { mat1 mat2 } {
    set ncol1 [llength $mat1]
    set nrow1 [llength [lindex $mat1 0]]

    set nrow2 [llength $mat2]
    set ncol2 [llength [lindex $mat2 0]]

    if { $ncol1 != $nrow2 } {
	error "error in math_ns::MatriuTranspPerMatriu. ncol1!=nrow2 ($ncol1!=$ncol2)"
    }
    set mat [create $nrow1 $ncol2]
    for { set i 0 } { $i < $nrow1 } { incr i } {
	for { set j 0 } { $j < $ncol2 } { incr j } {
	    set val 0.0
	    for { set k 0 } { $k < $ncol1 } { incr k } {
		set val [expr {$val+[lindex $mat1 $k $i]*[lindex $mat2 $k $j]}]
	    }
	    lset mat $i $j $val
	}
    }
    return $mat
}

proc math_ns::MatriuPerMatriuTransp { mat1 mat2 } {
    set nrow1 [llength $mat1]
    set ncol1 [llength [lindex $mat1 0]]

    set ncol2 [llength $mat2]
    set nrow2 [llength [lindex $mat2 0]]

    if { $ncol1 != $nrow2 } {
	error "error in math_ns::MatriuPerMatriuTransp. ncol1!=nrow2 ($ncol1!=$ncol2)"
    }
    set mat [create $nrow1 $ncol2]
    for { set i 0 } { $i < $nrow1 } { incr i } {
	for { set j 0 } { $j < $ncol2 } { incr j } {
	    set val 0.0
	    for { set k 0 } { $k < $ncol1 } { incr k } {
		set val [expr {$val+[lindex $mat1 $i $k]*[lindex $mat2 $j $k]}]
	    }
	    lset mat $i $j $val
	}
    }
    return $mat
}


proc math_ns::ScalarDotMatrix { sc mat } {
    set nrow [llength $mat]
    set ncol [llength [lindex $mat 0]]

    set retmat [create $nrow $ncol]
    for { set i 0 } { $i < $nrow } { incr i } {
	for { set j 0 } { $j < $ncol } { incr j } {
	    lset retmat $i $j [expr {$sc*[lindex $mat $i $j]}]
	}
    }
    return $retmat
}


















