
package require sqlite3
package require mather
package require ncgi

namespace eval rsdata {
    variable cache_euler_angles ""
}

if 0 {
    package require dde
    proc dd { args } {
	dde poke GIDDDE Project TclCommand $args
	dde request GIDDDE Project TclCommand
    }
    interp alias "" GiD_Info "" dd GiD_Info
}

proc rsdata::CalcJ { widthY widthZ } {
    set inum 12
    set TorsionFactor [list 1.0 1.5 2.0 2.5 3.0 4.0 5.0 6.0 7.0 8.0 \
	9.0 10.0]
    set  TorsionBeta [list 0.141 0.196 0.229 0.249 0.263 0.281 0.291 0.299 0.303 \
	    0.307 0.310 0.313 0.333]
    
    if { $widthY < $widthZ } {
	set b $widthZ; set e $widthY
    } else { set b $widthY; set e $widthZ; }
    set m [expr {1.0*$b/$e}]

    for { set i 1 } { $i < $inum } { incr i } {
	if { $m < ([lindex $TorsionFactor $i]+[lindex $TorsionFactor [expr {$i-1}]])/2.0 } {
	    set beta [expr {[lindex $TorsionBeta [expr {$i-1}]]}]
	    break
	}
    }
    if { $i == $inum } {
	if { $m < [lindex $TorsionFactor [exp {$inum-1}]] } {
	    set beta [lindex $TorsionBeta [expr {$inum-1}]]
	} else { set beta [lindex $TorsionBeta $inum] } 
    }
    return [expr {$beta*$b*$e*$e*$e}]
}

proc rsdata::unitsfactor { system_from system_to magnitude } {

    set factor 1.0
    switch $system_from {
	"N-m-kg" - m { set factor [expr {$factor*1.0}] }
	"N-mm-kg" - mm {
	    switch $magnitude {
		N { set factor [expr {$factor*1.0}] }
		m { set factor [expr {$factor*1e-3}] }
		N/m { set factor [expr {$factor*1e3}] }
		Nm { set factor [expr {$factor*1e-3}] }
		Pa { set factor [expr {$factor*1e6}] }
	    }
	}
	"Kp-cm-utm" - cm {
	    switch $magnitude {
		N { set factor [expr {$factor*9.8}] }
		m { set factor [expr {$factor*1e-2}] }
		N/m { set factor [expr {$factor*980.0}] }
		Nm { set factor [expr {$factor*0.098}] }
		Pa { set factor [expr {$factor*98000.0}] }
	    }
	}
	"kN-m-MPa-ton" {
	    switch $magnitude {
		N { set factor [expr {$factor*1.0e3}] }
		m { set factor [expr {$factor*1.0}] }
		N/m { set factor [expr {$factor*1e3}] }
		Nm { set factor [expr {$factor*1e3}] }
		Pa { set factor [expr {$factor*1e6}] }
	    }
	}
	default {
	    WarnWin "error in rsdata::unitsfactor: '$system_from' unknown"
	}
    }
    switch $system_to {
	"N-m-kg" - m { set factor [expr {$factor*1.0}] }
	"N-mm-kg" - mm {
	    switch $magnitude {
		N { set factor [expr {$factor*1.0}] }
		m { set factor [expr {$factor*1e3}] }
		N/m { set factor [expr {$factor*1e-3}] }
		Nm { set factor [expr {$factor*1e3}] }
		Pa { set factor [expr {$factor*1e-6}] }
	    }
	}
	"Kp-cm-utm" - cm {
	    switch $magnitude {
		N { set factor [expr {$factor/9.8}] }
		m { set factor [expr {$factor*1e2}] }
		N/m { set factor [expr {$factor/980.0}] }
		Nm { set factor [expr {$factor/0.098}] }
		Pa { set factor [expr {$factor/98000.0}] }
	    }
	}
	"kN-m-MPa-ton" {
	    switch $magnitude {
		N { set factor [expr {$factor*1.0e-3}] }
		m { set factor [expr {$factor*1.0}] }
		N/m { set factor [expr {$factor*1e-3}] }
		Nm { set factor [expr {$factor*1e-3}] }
		Pa { set factor [expr {$factor*1e-6}] }
	    }
	}
	default {
	    WarnWin "error in rsdata::unitsfactor: '$system_to' unknown"
	}
    }
    return $factor
}

proc rsdata::table_exists { name } {

    set ret [db eval { select name from sqlite_master where type='table' 
	    and name=$name }]
    if { ![llength $ret] } { return 0 } else { return 1 }
}

proc rsdata::delete_table_if_exits { name } {
    if { ![table_exists $name] } { return }
    db eval "drop table $name"
}

proc rsdata::write_file_do { filename } {

    sqlite3 db $filename
    if { [catch { db eval { PRAGMA synchronous = OFF } }] } {
	file delete $filename
	sqlite3 db $filename
	db eval { PRAGMA synchronous = OFF }
    }

    db function calc_euler_angle rsdata::calc_euler_angle
    db function calc_euler_angle_beam rsdata::calc_euler_angle_beam

    db eval { begin }

    delete_table_if_exits parameters
    db eval { create table parameters(name text unique,value text) }

    set units_system_to [GidUtils::GetProblemDataValue Results_units]
    set accepted_units_system [list N-m-kg N-cm-kg N-mm-kg Kp-cm-utm]
    if { [lsearch -exact $accepted_units_system $units_system_to] == -1 } {
	WarnWin [= "Result units %s are not accepted" $units_system_to]
    }
    set gravity [expr {9.8*[unitsfactor m $units_system_to m]}]
    db eval { insert into parameters values('gravity',$gravity) }

################################################################################
#    Nodes
################################################################################

    delete_table_if_exits nodes
    db eval { create table nodes(num integer primary key,x numeric,y numeric,z numeric,
	assambling integer) }

    set fac [unitsfactor [GidUtils::GetProblemDataValue Mesh_units] $units_system_to m]
    set numnodes 0
    foreach "num x y z" [GiD_Info Mesh Nodes] {
	foreach i [list x y z] { set $i [expr {$fac*[set $i]}] }
	db eval {insert into nodes(num,x,y,z) values($num,$x,$y,$z)}
	incr numnodes
    }
    db eval { insert into parameters values('numoriginalnodes',$numnodes) }
    db eval { insert into parameters values('numnodes',$numnodes) }

################################################################################
#    Triangle elements properties
################################################################################

    delete_table_if_exits elements_triangles
    db eval { create table elements_triangles(
	num integer primary key,material_type text,c1 numeric, c2 numeric, c3 numeric, mat int,
	euler1 numeric, euler2 numeric, euler3 numeric) }
    
    set matnum 0
    foreach i [GiD_Info conditions -localaxes Concrete_Shell mesh] {
	set elm [lindex $i 1]
	foreach "l1 l2 l3" [lindex $i 3] break
	if { $l1 eq "-Default-" } { set l1 "" }
	foreach "- c1 c2 c3 -" [GiD_Info Mesh Elements Triangle $elm] break
	set data [lrange $i 4 end]
	if { ![info exists concrete_shell($data)] } {
	    set concrete_shell($data) [incr matnum]
	    set mat $matnum
	}
	db eval {insert into elements_triangles values(
	    $elm,'cs',$c1,$c2,$c3,$mat,$l1,$l2,$l3)}        
    }
    delete_table_if_exits material_isotropic_shell
    db eval { create table material_isotropic_shell(num integer primary key,
	thickness numeric,specific_weight numeric,E numeric,nu numeric)}

    set facN [unitsfactor "N-mm-kg" $units_system_to N]
    set facM [unitsfactor "N-mm-kg" $units_system_to m]
    set facE [expr {$facN/($facM*$facM)}]
    set self_weight [expr {2.5e-05*$facN/($facM*$facM*$facM)}]
    set nu 0.2
    foreach i [array names concrete_shell] {
	foreach "units thickness concrete_type" $i break
	set fcm_N_mm2 [string range $concrete_type 3 end]
	set E [expr {$facE*8500.0*pow($fcm_N_mm2,1.0/3.0)}]

	set facM [unitsfactor $units $units_system_to m]
	set thickness [expr {$facM*$thickness}]
	db eval {insert into material_isotropic_shell values($concrete_shell($i),
	    $thickness,$self_weight,$E,$nu)}
    }
    array unset concrete_shell
    
    foreach i [GiD_Info conditions -localaxes Shell mesh] {
	set elm [lindex $i 1]
	foreach "l1 l2 l3" [lindex $i 3] break
	if { $l1 eq "-Default-" } { set l1 "" }
	foreach "- c1 c2 c3 -" [GiD_Info Mesh Elements Triangle $elm] break
	set data [lrange $i 4 5]
	eval lappend data [lrange $i 9 12]
	if { ![info exists shell($data)] } {
	    set shell($data) [incr matnum]
	    set mat $matnum
	}
	db eval {insert into elements_triangles values(
	    $elm,'cs',$c1,$c2,$c3,$mat,$l1,$l2,$l3)}        
    }
    foreach i [array names shell] {
	foreach "units thickness E nu - self_weight" $i break
	set facN [unitsfactor $units $units_system_to N]
	set facM [unitsfactor $units $units_system_to m]
	set facM2 [expr {$facM*$facM}]
	set facM3 [expr {$facM*$facM2}]
	set facNdm2 [expr {$facN/double($facM2)}]
	set facNdm3 [expr {$facN/double($facM3)}]

	foreach "n u" [list thickness facM E facNdm2 self_weight facNdm3] {
	    set $n [expr {[set $u]*[set $n]}]
	}        
	db eval {insert into material_isotropic_shell values($shell($i),
	    $thickness,$self_weight,$E,$nu)}
    }
    array unset shell

################################################################################
#    Beam elements properties
################################################################################

    delete_table_if_exits elements_beams
    db eval { create table elements_beams(
	num integer primary key,material_type text,c1 numeric, c2 numeric, mat int,
	euler1 numeric, euler2 numeric, euler3 numeric) }

    db eval { insert into parameters values('num_beam_element_min','') }
    db eval { insert into parameters values('num_beam_element_max','') }

    db eval { create trigger count_elements_beams insert on elements_beams
	begin
	update parameters set value=new.num where name='num_beam_element_min' and
	(value='' or value > new.num) ;
	update parameters set value=new.num where name='num_beam_element_max' and
	(value='' or value < new.num) ;
	end
    }

    set matnum 0
    foreach i [GiD_Info conditions -localaxes Section mesh] {
	set elm [lindex $i 1]
	foreach "l1 l2 l3" [lindex $i 3] break
	if { $l1 eq "-Default-" } { set l1 "" }
	foreach "- c1 c2 -" [GiD_Info Mesh Elements Linear $elm] break
	set data [lrange $i 4 end]
	if { ![info exists Section($data)] } {
	    set Section($data) [incr matnum]
	    set mat $matnum
	} else { set mat $Section($data) }
	db eval {insert into elements_beams values(
	    $elm,'s',$c1,$c2,$mat,$l1,$l2,$l3)}        
    }
    foreach i [GiD_Info conditions -localaxes Rectangular_Section mesh] {
	set elm [lindex $i 1]
	foreach "l1 l2 l3" [lindex $i 3] break
	if { $l1 eq "-Default-" } { set l1 "" }
	foreach "- c1 c2 -" [GiD_Info Mesh Elements Linear $elm] break
	foreach "units widthy widthz G E sw" [lrange $i 4 end] break
	set J [CalcJ $widthy $widthz]
	set A [expr {$widthy*$widthz}]
	set Iy [expr {1.0/12.0*$widthy*$widthz*$widthz*$widthz}]
	set Iz [expr {1.0/12.0*$widthz*$widthy*$widthy*$widthy}]
	set data [list rectangular $units $A $Iy $Iz $J $G $E $sw \
		[expr {2.0*$Iy/$widthz}] [expr {2.0*$Iz/$widthy}] \
		$A $A 0.0]
	if { ![info exists Section($data)] } {
	    set Section($data) [incr matnum]
	    set mat $matnum
	} else { set mat $Section($data) }
	db eval {insert into elements_beams values(
	    $elm,'s',$c1,$c2,$mat,$l1,$l2,$l3)}        
    }

    delete_table_if_exits material_beam
    db eval { create table material_beam(num integer primary key,
	Area numeric,Inertia_y numeric,Inertia_z numeric,J numeric,
	G numeric,E numeric,Specific_weight numeric,
	W_y numeric,W_z numeric,Aty numeric,Atz numeric,Maximum_stress numeric)}

    foreach i [array names Section] {
	foreach "Name units Area Inertia_y Inertia_z J G E Specific_weight \
	    W_y W_z Aty Atz Maximum_stress" $i break

	set facN [unitsfactor $units $units_system_to N]
	set facM [unitsfactor $units $units_system_to m]
	set facM2 [expr {$facM*$facM}]
	set facM3 [expr {$facM*$facM2}]
	set facM4 [expr {$facM2*$facM2}]
	set facNdm2 [expr {$facN/double($facM2)}]
	set facNdm3 [expr {$facN/double($facM3)}]

	foreach "n u" [list Area facM2 Inertia_y facM4 Inertia_z facM4 J facM4 \
		G facNdm2 E facNdm2 Specific_weight facNdm3 \
		W_y facM3 W_z facM3 Aty facM2 Atz facM2 Maximum_stress facNdm2] {
	    set $n [expr {[set $u]*[set $n]}]
	}
	db eval {insert into material_beam values($Section($i),
	    $Area,$Inertia_y,$Inertia_z,$J,$G,$E,$Specific_weight,
	    $W_y,$W_z,$Aty,$Atz,$Maximum_stress)}
    }
    array unset Section

################################################################################
#    Constraints
################################################################################

    delete_table_if_exits constraints
    db eval { create table constraints(
	num integer,c1 numeric, c2 numeric, c3 numeric,
	c4 numeric, c5 numeric, c6 numeric,
	v1 numeric, v2 numeric, v3 numeric,
	v4 numeric, v5 numeric, v6 numeric,
	euler1 numeric, euler2 numeric, euler3 numeric) }

    set ret [GiD_Info conditions Surface_Constraints Mesh]
    eval lappend ret [GiD_Info conditions Line_Constraints Mesh]
    eval lappend ret [GiD_Info conditions Point_Constraints Mesh]
    foreach i $ret {
	set node [lindex $i 1]
	foreach "l1 l2 l3" [lindex $i 3] break
	foreach "c1 c2 c3 c4 c5 c6 units" [lrange $i 4 10] break
	set fac [unitsfactor $units $units_system_to m]
	foreach "v1 v2 v3 v4 v5 v6" [lrange $i 11 16] break
	foreach i [list v1 v2 v3] {
	    set $i [expr {$fac*[set $i]}]
	}
	db eval {insert into constraints values($node,$c1,$c2,$c3,$c4,$c5,$c6, 
	    $v1,$v2,$v3,$v4,$v5,$v6,$l1,$l2,$l3)}
    }

    delete_table_if_exits beam_elem_constraints
    db eval { create table beam_elem_constraints(
	num integer,c1 numeric, c2 numeric, c3 numeric,
	c4 numeric, c5 numeric, c6 numeric,
	v1 numeric, v2 numeric, v3 numeric,
	v4 numeric, v5 numeric, v6 numeric,
	euler1 numeric, euler2 numeric, euler3 numeric) }

    foreach i [GiD_Info conditions Line_Elastic_Constraints Mesh] {
	set elm [lindex $i 1]
	foreach "l1 l2 l3" [lindex $i 3] break
	foreach "c1 c2 c3 c4 c5 c6 units" [lrange $i 4 10] break
	set facN [unitsfactor $units $units_system_to N]
	set facM [unitsfactor $units $units_system_to m]

	foreach "v1 v2 v3 v4 v5 v6" [lrange $i 11 16] break
	foreach i [list v1 v2 v3] {
	    set $i [expr {[set $i]*$facN/($facM*$facM)}]
	}
	foreach i [list v4 v5 v6] {
	    set $i [expr {[set $i]*$facN}]
	}
	foreach i [list 1 2 3 4 5 6] {
	    if { [set c$i] } { set c$i 2 }
	}
	db eval {insert into beam_elem_constraints values($elm,$c1,$c2,$c3,$c4,$c5,$c6, 
	    $v1,$v2,$v3,$v4,$v5,$v6,$l1,$l2,$l3)}
    }

    delete_table_if_exits triangle_elem_constraints
    db eval { create table triangle_elem_constraints(
	num integer,c1 numeric, c2 numeric, c3 numeric,
	c4 numeric, c5 numeric, c6 numeric,
	v1 numeric, v2 numeric, v3 numeric,
	v4 numeric, v5 numeric, v6 numeric,
	euler1 numeric, euler2 numeric, euler3 numeric) }

    foreach i [GiD_Info conditions Surface_Elastic_Constraints Mesh] {
	set elm [lindex $i 1]
	foreach "l1 l2 l3" [lindex $i 3] break
	foreach "c1 c2 c3 c4 c5 c6 units" [lrange $i 4 10] break
	set facN [unitsfactor $units $units_system_to N]
	set facM [unitsfactor $units $units_system_to m]

	foreach "v1 v2 v3 v4 v5 v6" [lrange $i 11 16] break
	foreach i [list v1 v2 v3] {
	    set $i [expr {[set $i]*$facN/($facM*$facM*$facM)}]
	}
	foreach i [list v4 v5 v6] {
	    set $i [expr {[set $i]*$facN/$facM}]
	}
	foreach i [list 1 2 3 4 5 6] {
	    if { [set c$i] } { set c$i 2 }
	}
	db eval {insert into triangle_elem_constraints values($elm,$c1,$c2,$c3,$c4,$c5,$c6, 
	    $v1,$v2,$v3,$v4,$v5,$v6,$l1,$l2,$l3)}
    }


################################################################################
#    functions
################################################################################

    delete_table_if_exits functions
    db eval { create table functions(name text primary key,func text) }

    foreach mat [GiD_Info materials] {
	if { [GiD_Info materials $mat BOOK] eq "Tables" } {
	    set vals [lrange [lindex [GiD_Info materials $mat] 4] 2 end-1]
	    set func "interpolate_table \[P deltaT\] {$vals}"
	    set name tables_$mat
	    db eval { insert into functions values($name,$func) }
	} elseif { [GiD_Info materials $mat BOOK] eq "Custom_data" } {
	    set name custom_data_$mat
	    set func [ncgi::decode [lindex [GiD_Info materials $mat] 6]]
	    set units [lindex [GiD_Info materials  $mat] 4]
	    set facN [unitsfactor $units $units_system_to N]
	    set facM [unitsfactor $units $units_system_to m]
	    set func "set facN $facN\nset facM $facM\n$func"
	    db eval { insert into functions values($name,$func) }
	}
    }

################################################################################
#    Loads
################################################################################

    delete_table_if_exits load_element
    db eval { create table load_element(num integer,
	pressure1_x numeric,pressure1_y numeric,pressure1_z numeric,
	pressure1_mx numeric,islocal integer,loadcase text,function text) }
    
    delete_table_if_exits load_node
    db eval { create table load_node(num integer,
	force_x numeric,force_y numeric,force_z numeric,
	momentum_x numeric,momentum_y numeric,momentum_z numeric,
	loadcase text,function text) }

    delete_table_if_exits custom_load_elem

    set numintervals [lindex [GiD_Info intvdata num] 1]
    for { set interval 2 } { $interval <= $numintervals } { incr interval } {
	set loadcase [lindex [GiD_Info intvdata -interval $interval] 2]

	foreach i [GiD_Info conditions -interval $interval Global_Shell_Load Mesh] {
	    set node [lindex $i 1]
	    set units [lindex $i 3]
	    foreach "px py pz" [lrange $i 4 6] break
	    set facN [unitsfactor $units $units_system_to N]
	    set facM [unitsfactor $units $units_system_to m]
	    set facP [expr {$facN/($facM*$facM)}]
	    foreach i [list px py pz] {
		set $i [expr {$facP*[set $i]}]
	    }
	    db eval {insert into load_element values($node,$px,$py,$pz,0.0,0,$loadcase,'')}
	}
	foreach i [GiD_Info conditions -interval $interval Global_Beam_Load Mesh] {
	    set node [lindex $i 1]
	    set units [lindex $i 3]
	    foreach "px py pz" [lrange $i 4 6] break
	    set facN [unitsfactor $units $units_system_to N]
	    set facM [unitsfactor $units $units_system_to m]
	    set facP [expr {$facN/$facM}]
	    foreach i [list px py pz] {
		set $i [expr {$facP*[set $i]}]
	    }
	    db eval {insert into load_element values($node,$px,$py,$pz,0.0,0,$loadcase,'')}
	}
	foreach i [GiD_Info conditions -interval $interval Local_Beam_Load Mesh] {
	    set node [lindex $i 1]
	    set units [lindex $i 3]
	    foreach "px py pz mx" [lrange $i 4 7] break
	    set facN [unitsfactor $units $units_system_to N]
	    set facM [unitsfactor $units $units_system_to m]
	    set facP [expr {$facN/$facM}]
	    foreach i [list px py pz] {
		set $i [expr {$facP*[set $i]}]
	    }
	    set mx [expr {$facN*$mx}]
	    db eval {insert into load_element values($node,$px,$py,$pz,$mx,1,$loadcase,'')}
	}
	set gravity ""
	foreach axe [list X Y Z] {
	    lappend gravity [GidUtils::GetProblemDataValue Gravity_$axe]
	}
	set gravity [math::Normalizevector $gravity]
	
	foreach i [GiD_Info conditions -interval $interval Self_weight_Beam Mesh] {
	    set elm [lindex $i 1]
	    set ret [db eval {
		    select m.Area,m.Specific_weight
		    from elements_beams as e
		    join material_beam as m on m.num=e.mat
		    where e.num=$elm
		}]
	    foreach "area sw" $ret break
	    set p [math::ScalarDotVector [expr {$sw*$area}] $gravity]
	    foreach "px py pz" $p break
	    db eval {insert into load_element values($elm,$px,$py,$pz,0.0,0,$loadcase,'')}
	}
		
	foreach i [GiD_Info conditions -interval $interval Punctual_Load Mesh] {
	    set node [lindex $i 1]
	    set units [lindex $i 3]
	    foreach "px py pz mx my mz" [lrange $i 4 9] break
	    set facN [unitsfactor $units $units_system_to N]
	    set facM [unitsfactor $units $units_system_to m]
	    set facMom [expr {$facN*$facM}]
	    foreach i [list px py pz] {
		set $i [expr {$facN*[set $i]}]
	    }
	    foreach i [list mx my mz] {
		set $i [expr {$facMom*[set $i]}]
	    }
	    db eval {insert into load_node values($node,$px,$py,
		$pz,$mx,$my,$mz,$loadcase,'')}
	}
	
	foreach i [GiD_Info conditions -interval $interval Dynamic-Point-Load Mesh] {
	    set node [lindex $i 1]
	    set units [lindex $i 3]
	    foreach "px py pz mx my mz funtype - - - - table" [lrange $i 4 15] break
	    set facN [unitsfactor $units $units_system_to N]
	    set facM [unitsfactor $units $units_system_to m]
	    set facMom [expr {$facN*$facM}]
	    foreach i [list px py pz] {
		set $i [expr {$facN*[set $i]}]
	    }
	    foreach i [list mx my mz] {
		set $i [expr {$facMom*[set $i]}]
	    }
	    if { $funtype eq "Table" } {
		set name tables_$table
		db eval {insert into load_node values($node,$px,$py,
		    $pz,$mx,$my,$mz,$loadcase,$name)}
	    }
	}

	set ret [GiD_Info conditions -interval $interval Line_Custom_Load Mesh]
	if { [llength $ret] } {
	    if { ![table_exists custom_load_elem] } {
		db eval { create table custom_load_elem(num integer,
		    loadcase text,function text) }
	    }
	    foreach i $ret {
		set elem [lindex $i 1]
		set func custom_data_[lindex $i 3]
		db eval {insert into custom_load_elem values($elem,$loadcase,$func)}
	    }
	}
    }
    
################################################################################
#    Detailed results
################################################################################

    delete_table_if_exits detailed_results
    set ret [GiD_Info conditions Detailed_results Mesh]
    if { [llength $ret] } {
	db eval { create table detailed_results(num integer,
	    name text,loadcase text,step numeric,
	    disp_x numeric,disp_y numeric,disp_z numeric) }
	
	foreach i $ret {
	    set node [lindex $i 1]
	    set name [lindex $i 3]
	    db eval {insert into detailed_results (num,name) values($node,$name)}
	}
    }

################################################################################
#    Rotules
################################################################################

    set ret [GiD_Info conditions Rotules Mesh]
    eval lappend ret [GiD_Info conditions Point_disconnect_degrees Mesh]

    delete_table_if_exits connection_constraints
    if { [llength $ret] } {
	db eval { create table connection_constraints(
	    num integer,connected_num integer,c1 numeric, c2 numeric, c3 numeric,
	    c4 numeric, c5 numeric, c6 numeric) }
    }
    foreach i $ret {
	set node [lindex $i 1]
	if { [llength $i] == 3 } {
	    foreach "id c1 c2 c3 c4 c5 c6" [list "" 1 1 1 0 0 0] break
	} else {
	    foreach "id c1 c2 c3 c4 c5 c6" [lrange $i 4 10] break
	    foreach j [list c1 c2 c3 c4 c5 c6] {
		set $j [expr {([set $j])?0:1}]
	    }
	}
	foreach "x y z" [db eval { select x,y,z from nodes where num=$node }] break

	set ret [db eval { select num,c1,c2 from elements_beams
		where c1=$node or c2=$node
	    }]
	if { $id ne "" } {
	    set elms ""
	    foreach j [GiD_Info conditions Line_disconnect_id Mesh] {
		if { [lindex $j 3] eq $id } {
		    lappend elms [lindex $j 1]
		}
	    }
	    set newret ""
	    set elms [lsort -integer $elms]
	    foreach "num n1 n2" $ret {
		if { [lsearch -integer -sorted $elms $num] != -1 } {
		    lappend newret $num $n1 $n2
		}
	    }
	    if { [llength $newret] < [llength $ret] } {
		set ret $newret
	    } else {
		set ret [lrange $newret 3 end]
	    }
	} else { set ret [lrange $ret 3 end] }
	foreach "num n1 n2" $ret {
	    db eval { insert into nodes(x,y,z) values ($x,$y,$z) }
	    db eval { update parameters set value=value+1 where name='numnodes' }

	    set newnode [db last_insert_rowid]
	    if { $node == $n1 } {
		db eval { update elements_beams set c1=$newnode where
		    num=$num
		}
	    } else {
		db eval { update elements_beams set c2=$newnode where
		    num=$num
		}
	    }
	    db eval { insert into connection_constraints values(
		$newnode,$node,$c1,$c2,$c3,$c4,$c5,$c6) }
	}
    }

################################################################################
#    loadcases
################################################################################

    delete_table_if_exits combined_loadcases
    db eval {
	create table combined_loadcases(name text,
	elu integer,loadcase text,factor numeric)
    }
    if { [GidUtils::GetProblemDataValue Activate_Load_Cases] ne "none" } {
	set clc [GidUtils::GetProblemDataValue Combined_load_cases]
	foreach "cl elu ls" [lrange $clc 2 end] {
	    foreach "f name" [split $ls ,] {
		db eval {
		    insert into combined_loadcases values(
		    $cl,$elu,$name,$f)
		}
	    }
	}
    } else {
	set numintervals [lindex [GiD_Info intvdata num] 1]
	for { set interval 2 } { $interval <= $numintervals } { incr interval } {
	    set loadcase [lindex [GiD_Info intvdata -interval $interval] 2]
	    db eval {
		insert into combined_loadcases values(
		'Static',1,$loadcase,1.0)
	    }
	}
    }

################################################################################
#    calculate Euler Angles for default values
################################################################################

    db eval {
	insert or replace into elements_triangles select
	e.num,e.material_type,e.c1,e.c2,e.c3,e.mat,
	calc_euler_angle(1,e.num,n1.x,n1.y,n1.z,n2.x,n2.y,n2.z,n3.x,n3.y,n3.z),
	calc_euler_angle(2,e.num,n1.x,n1.y,n1.z,n2.x,n2.y,n2.z,n3.x,n3.y,n3.z),
	calc_euler_angle(3,e.num,n1.x,n1.y,n1.z,n2.x,n2.y,n2.z,n3.x,n3.y,n3.z)
	from elements_triangles as e
	join nodes as n1 on e.c1 = n1.num
	join nodes as n2 on e.c2 = n2.num
	join nodes as n3 on e.c3 = n3.num
	where e.euler1 = ''
    }
    db eval {
	insert or replace into elements_beams select
	e.num,e.material_type,e.c1,e.c2,e.mat,
	calc_euler_angle_beam(1,e.num,n1.x,n1.y,n1.z,n2.x,n2.y,n2.z),
	calc_euler_angle_beam(2,e.num,n1.x,n1.y,n1.z,n2.x,n2.y,n2.z),
	calc_euler_angle_beam(3,e.num,n1.x,n1.y,n1.z,n2.x,n2.y,n2.z)
	from elements_beams as e
	join nodes as n1 on e.c1 = n1.num
	join nodes as n2 on e.c2 = n2.num
	where e.euler1 = ''
    }
    
################################################################################
#    calculate Assamble positions
################################################################################
    
    # dim can be: 3,5,6
    db eval { create temp table nodeconecs(num integer primary key,dim numeric,
	vx numeric,vy numeric,vz numeric) }
    
    db eval { select c1,c2,c3,euler1,euler2,euler3 from elements_triangles } {
	foreach "vx vy vz" [give_triangle_normal $euler1 $euler2 $euler3] break
	foreach i [list c1 c2 c3] {
	    set num [set $i]
	    db eval { update or ignore nodeconecs set dim=5,vx=$vx,vy=$vy,vz=$vz
		where num=$num and dim < 5
	    }
	    if { ![db changes] } {
		db eval { update or ignore nodeconecs set dim=6
		    where num=$num and dim=5 and
		    abs(vx*$vx+vy*$vy+vz*$vz)>$cosangle
		}
	    }
	    # WARNING: temporaly set to 6. Should be 5
	    if { ![db changes] } {
		db eval { insert or ignore into nodeconecs(num,dim,vx,vy,vz) values
		    ($num,6,$vx,$vy,$vz)
		}
	    }
	}
    }
    db eval { select c1,c2 from elements_beams } {
	foreach i [list c1 c2] {
	    set num [set $i]
	    db eval { update or ignore nodeconecs set dim=6
		where num=$num and dim < 6
	    }
	    if { ![db changes] } {
		db eval { insert or ignore into nodeconecs(num,dim) values($num,6) }
	    }
	}
    }

    set pos 1
    db eval { select num,dim from nodeconecs } {
	db eval { update nodes set assambling=$pos where num=$num }
	incr pos $dim
    }
    incr pos -1
    db eval { insert into parameters values('solversize',$pos) }

    db eval { drop table nodeconecs }

    db eval { commit }
    db close
}

proc rsdata::give_triangle_normal { euler1 euler2 euler3 } {

    set T [math::GiveMatrixFromEuler [list $euler1 $euler2 $euler3]]
    return [math::GetAxeInMatrix $T z]
}

proc rsdata::_calc_euler_angles { num v1 v2 v3 } {
    variable cache_euler_angles

    set kprima [math::VectorProd [math::VectorDiff $v2 $v1] [math::VectorDiff $v3 $v1]]
    set kprima [math::Normalizevector $kprima]
    set m [math::CalcMatGivenZAxe $kprima]

    set cache_euler_angles $num
    lappend cache_euler_angles [math::GiveEulerAnglesFromMatrix $m]
}

# ea_num = 1,2,3
proc rsdata::calc_euler_angle { ea_num elmnum x1 y1 z1 x2 y2 z2 x3 y3 z3 } {
    variable cache_euler_angles

    if { [lindex $cache_euler_angles 0] != $elmnum } {
	_calc_euler_angles $elmnum [list $x1 $y1 $z1] [list $x2 $y2 $z2] \
	    [list $x3 $y3 $z3]
    }
    return [lindex $cache_euler_angles 1 [expr {$ea_num-1}]]
}

proc rsdata::_calc_euler_angles_beam { num v1 v2 } {
    variable cache_euler_angles

    set iprima [math::VectorDiff $v2 $v1]
    set iprima [math::Normalizevector $iprima]
   
    if { abs([lindex $iprima 0]) > $math::epsilon || \
	abs([lindex $iprima 1]) > $math::epsilon } {
	if { abs([lindex $iprima 0]) > $math::epsilon } {
	    set jprima [list [expr {-1.0*[lindex $iprima 1]/[lindex $iprima 0]}] \
		    1.0 0.0]
	} else {
	    set jprima [list 1.0 0.0 0.0]
	}
	set jprima [math::Normalizevector $jprima]
	set kprima [math::VectorProd $iprima $jprima]
	if { [lindex $kprima 2] < 0.0 } {
	    set jprima [math::ScalarDotVector -1.0 $jprima]
	    set kprima [math::VectorProd $iprima $jprima]
	}
    } else {
	set jprima [list 1.0 0.0 0.0]
	set kprima [math::VectorProd $iprima $jprima]
    }
    set m [math::Identity 3]
    set m [math::SetAxeInMatrix $m x $iprima]
    set m [math::SetAxeInMatrix $m y $jprima]
    set m [math::SetAxeInMatrix $m z $kprima]

    set cache_euler_angles $num
    lappend cache_euler_angles [GiveEulerAnglesFromMatrix $m]
}

# ea_num = 1,2,3
proc rsdata::calc_euler_angle_beam { ea_num elmnum x1 y1 z1 x2 y2 z2 } {
    variable cache_euler_angles

    if { [lindex $cache_euler_angles 0] != $elmnum } {
	_calc_euler_angles_beam $elmnum [list $x1 $y1 $z1] [list $x2 $y2 $z2]
    }
    return [lindex $cache_euler_angles 1 [expr {$ea_num-1}]]
}


proc rsdata::write_file { { andcalc "" } } {
    set fileb [lindex [GiD_Info Project] 1]
    set file [file join $fileb.gid [file tail $fileb].dsq]
    rsdata::write_file_do $file
    if { $andcalc eq "" } {
	tk_messageBox -message "Created file $file"
    } else {
	set ::env(TCLLIBPATH) [list ~/mytcltk /tcltk {C:\TclTk\ActiveTcl8.5\lib}]
	set exec {C:\TclTk\TclTk8.5a2\bin\tclsh.exe}
	set exramseries {C:\Documents and Settings\ramsan\Mis documentos\barres\experimental\ramseries_main.tcl}
	set resfile [file root $file].flavia.res
	file delete $resfile
	exec $exec $exramseries $file
	if { [file exists $resfile] && [file size $resfile] > 0 } {
	    dialogwin_snit .gid._ask -title [= "Calculation OK"] \
		-cancelname [= Postprocess] \
		-entrytext [_ "Calculation finished OK"]
	    set action [.gid._ask createwindow]
	    destroy .gid._ask
	    if { $action == 0 } {
		GiD_Process escape escape escape escape Postprocess
	    }
	} else {
	    tk_messageBox -message "Calculation failed"
	}
    }
}










