

namespace eval shell_sections {
    variable sections ""
}

proc shell_sections::Create { f data } {
    variable tablelist
    variable sections
    global ProblemTypePriv

    label $f.lsize -text [= "Enter sections defined by two points to integrate over shell parts:"]
    set sw [ScrolledWindow $f.lf -relief sunken -borderwidth 0 -scrollbar vertical]
    set tablelist [tablelist::tablelist $sw.lb -width 45 \
		       -exportselection 0 \
		       -columns [list \
		                     15  [= Name]      left \
		                     15 [= "Point 1"]     left \
		                     15  [= "Point 2"]     left \
		                    ] \
		       -labelcommand tablelist::sortByColumn \
		       -background white \
		       -selectbackground navy -selectforeground white \
		       -stretch all -selectmode extended \
		       -highlightthickness 0 \
		       -listvariable shell_sections::sections]
    
    $sw setwidget $sw.lb

    button $f.erase -image $ProblemTypePriv(loadcaseimgerase-cross) -relief flat \
	-overrelief raised -command [namespace code [list deletesections $sw.lb]]
    DynamicHelp::register $f.erase balloon [= "Delete section"]
    
    set values ""
    set sections ""
    foreach "name x1 y1 z1 x2 y2 z2" [lrange $data 2 end] {
	set good 1
	foreach i "x1 y1 z1 x2 y2 z2" {
	    if { ![string is double -strict [set $i]] } {
		set good 0
		break
	    }
	}
	if { $good } {
	    lappend values $name
	    lappend sections [list $name $x1,$y1,$z1 $x2,$y2,$z2]
	}
    }
    
    labelframe $f.lb -text [= "Add shell section"]
    label  $f.lb.l -text [= "Name:"]
    ComboBox $f.lb.cb -textvariable shell_sections::name -values $values
    label $f.lb.l2 -text [= "Point 1:"]
    entry $f.lb.e1 -textvariable shell_sections::point1
    button $f.lb.b1 -text [= "Pick"] -padx 5 -pady 0 -relief flat -overrelief raised \
	-command [namespace code [list GetPoint $f.lb.b1 shell_sections::point1]]
    label $f.lb.l3 -text [= "Point 2:"]
    entry $f.lb.e2 -textvariable shell_sections::point2
    button $f.lb.b2 -text [= "Pick"] -padx 5 -pady 0 -relief flat -overrelief raised \
	-command [namespace code [list GetPoint $f.lb.b2 shell_sections::point2]]
    button $f.lb.b3 -text [= "Add"] -padx 20 -command \
	[namespace code [list AddSection $f.lb.b3]]

    $f.lb.cb bind <Return> [list $f.lb.b3 invoke]
    foreach i [list $f.lb.e1 $f.lb.e2] {
	bind $i <Return> [list $f.lb.b3 invoke]
    }
    bind [$sw.lb bodypath] <Double-1> [namespace code [list FromTL2entries $sw.lb]] 

    bind [$sw.lb bodypath] <ButtonPress-3> \
	[namespace code [list ContextualSubmenuStart $sw.lb %W %x %y]]
    bind [$sw.lb bodypath] <ButtonRelease-3> \
	[namespace code [list ContextualSubmenu $sw.lb %X %Y]] 
    grid $f.lb.l  $f.lb.cb - -sticky w
    grid $f.lb.l2 $f.lb.e1 $f.lb.b1 -sticky w
    grid $f.lb.l3 $f.lb.e2 $f.lb.b2 -sticky w
    grid $f.lb.b3 - - -pady 5 -padx 5 -sticky w

    grid configure $f.lb.b1 $f.lb.b2 -padx 3
    grid configure $f.lb.cb $f.lb.e1 $f.lb.e2 -sticky ew
    grid columnconfigure $f.lb 1 -weight 1

    tk::TabToWindow $f.lb.cb

    grid $f.lsize -sticky w
    grid $sw -sticky nsew
    grid $f.erase -sticky nw
    grid $f.lb -sticky nwew -pady 5
    grid rowconfigure $f 1 -weight 1
    grid columnconfigure $f 0 -weight 1


}

proc shell_sections::ContextualSubmenuStart { listbox w x y } {

    set tablelist::x [expr {$x + [winfo x $w]}]
    set tablelist::y [expr {$y + [winfo y $w]}]
 
    set idx [$listbox index @$tablelist::x,$tablelist::y]
    if { [lsearch [$listbox curselection] $idx] == -1 } {
	$listbox selection clear 0 end
	$listbox selection set $idx
    }
}

proc shell_sections::ContextualSubmenu { listbox X Y } {

    catch { destroy $listbox.menu }
    set menu [menu $listbox.menu]
    $menu add command -label [= Delete] -command [list shell_sections::deletesections $listbox]
    tk_popup $menu $X $Y
}

proc shell_sections::deletesections { listbox } {
    variable sections

    set new_sections ""
    for { set i 0 } { $i < [$listbox size] } { incr i } {
	if { ![$listbox selection includes $i] } {
	    lappend new_sections [lindex $sections $i]
	}
    }
    set sections $new_sections
}

proc shell_sections::FromTL2entries { listbox } {
    variable name
    variable point1
    variable point2

    foreach "name point1 point2" [$listbox get active] break
}

proc shell_sections::AddSection { b } {
    variable name
    variable point1
    variable point2
    variable sections

    set name [string trim $name]

    if { $name eq "" } {
	WarnWin [= "Section should have a name"] $b
	return
    }
    foreach i $sections {
	set old_name [lindex $i 0]
	if { $name eq $old_name } {
	    WarnWin [= "Section name '%s' already exists" $name] $b
	    return
	}
    }

    foreach i [list 1 2] {
	if { ![regexp {^\s*(\S+)(?:\s+|,)(\S+)(?:\s+|,)(\S+)\s*$} [set point$i] {} px($i) \
		   py($i) pz($i)] || ![string is double -strict $px($i)] || \
		 ![string is double -strict $py($i)] || \
		 ![string is double -strict $pz($i)] } {
	    WarnWin [= "Point '%d' is not correct" $i] $b
	    return
	}
    }
    lappend sections [list $name $px(1),$py(1),$pz(1) $px(2),$py(2),$pz(2)]
    
}

proc shell_sections::GetPoint { button variable } {

    set w [winfo toplevel $button]
    set x [expr {[winfo rootx $button]+40}]
    set y [winfo rooty $button]
    wm withdraw $w
    toplevel $w.tmp
    wm geometry $w.tmp +$x+$y
    bind $w.tmp <Destroy> "[list GiD_Process escape] ; break"

    pack [button $w.tmp.l -text [= "Pick here to cancel"] -relief flat -overrelief raise \
	      -command [list GiD_Process escape]]

    set pnt [GidUtils::GetCoordinates [= "Enter point to define section"] NoJoin]

    if { [winfo exists $w.tmp] } {
	bind $w.tmp <Destroy> ""
	destroy $w.tmp
    }
    wm deiconify $w
	  
    if { $pnt ne "" } { set $variable $pnt }
}

proc shell_sections::ComunicateWithGiD { op args } {
    variable sections

    switch $op {
	"INIT" {
	    set PARENT [lindex $args 0]
	    upvar      [lindex $args 1] ROW
	    set GDN [lindex $args 2]
	    set STRUCT [lindex $args 3]

	    set f [frame $PARENT.f]

	    shell_sections::Create $f [DWLocalGetValue $GDN $STRUCT Shell_integration_sections]

	    #GidHelpRecursive $f [= "Define sections in order to calculate its integrals over shell elements"]
	    grid $f -row $ROW -column 0 -sticky nsew -columnspan 2 -pady 3 -padx 2
	    grid rowconf $PARENT $ROW -weight 1
	    grid columnconf $PARENT 1 -weight 1
	    return ""
	}
	"SYNC" {
	    set GDN [lindex $args 0]
	    set STRUCT [lindex $args 1]

	    if { ![llength $sections] } {
		set val "#N# 1 -"
	    } else {
		set val [list #N# [expr {[llength $sections]*7}]]
		foreach i $sections {
		    regsub -all {\s+} [lindex $i 0] {_} name
		    lappend val $name
		    eval lappend val [split [lindex $i 1] ,]
		    eval lappend val [split [lindex $i 2] ,]
		}
	    }
	    DWLocalSetValue $GDN $STRUCT Shell_integration_sections $val
	    return ""
	}
    }
}

namespace eval ProblemDataWidget {
    variable bshape ""
    variable A_type
    variable boton ""
    variable control ""
}

proc ProblemDataWidget::Analysis_windows { op args } {
    variable bshape
    variable A_type
    switch $op {
	"INIT" {
	    set PARENT [lindex $args 0]
	    upvar [lindex $args 1] ROW
	    set GDN [lindex $args 2]
	     set STRUCT [lindex $args 3]
	    set QUESTION [lindex $args 4]

	    set bshape [Button $PARENT.bshape -justify center \
		-command "ProblemDataWidget::Comando" \
		-text "Analysis Preferences" \
		 -helptext "Define las caracteristicas generales del analisis" ]
	    grid $bshape -row [expr $ROW+1] -column 1 -sticky nw -pady 3 -padx 2
	   
	    if { $A_type == "Linear_Static" } {
	      grid remove $bshape
	    } else {
	      grid $bshape
	    }
	    return ""
	}
	 "SYNC" {
	     set GDN [lindex $args 0]
	     set STRUCT [lindex $args 1]
	     set QUESTION [lindex $args 2]
	     return ""
	}

	 "DEPEND" {
	     set GDN [lindex $args 0]
	     set STRUCT [lindex $args 1]
	     set QUESTION [lindex $args 2]
	     set ACTION [lindex $args 3]
	     set A_type [lindex $args 4]
	     
	     if { $bshape == "" } {
		return ""
	     }
	     if { $A_type == "Linear_Static" } {
		catch {grid remove $bshape}
	     } else {
		catch {grid $bshape}
	     }
	     return ""
	}
	"CLOSE" {

	}           
    }
}

proc ProblemDataWidget::Comando {} {
    variable A_type
    if {$A_type == "Linear_Static" } {
       return
    } else {
       GidOpenProblemData $A_type
    }
}


namespace eval MaterialWidget {
    variable const_type ""
    variable boton ""
}

proc MaterialWidget::Constitutive_Window { op args } {
    variable boton
    variable const_type
    switch $op {
	"INIT" {
	    set PARENT [lindex $args 0]
	    upvar [lindex $args 1] ROW
	    set GDN [lindex $args 2]
	     set STRUCT [lindex $args 3]
	    set QUESTION [lindex $args 4]

	    set boton [Button $PARENT.boton -justify center \
		-command "MaterialWidget::Comando"\
		-text "Edit constitutive model" \
		 -helptext "Define the constitutive model for the material" ]
	    set ROW [expr $ROW+1]
	    grid $boton -row $ROW -column 1 -sticky nw -pady 3 -padx 2
	   
	    if { $const_type == "Linear_Elastic" } {
	      grid remove $boton
	    } else {
	      grid $boton
	    }
	    return ""
	}
	 "SYNC" {
	     set GDN [lindex $args 0]
	     set STRUCT [lindex $args 1]
	     set QUESTION [lindex $args 2]
	     return ""
	}

	 "DEPEND" {
	     set GDN [lindex $args 0]
	     set STRUCT [lindex $args 1]
	     set QUESTION [lindex $args 2]
	     set ACTION [lindex $args 3]
	     set const_type [lindex $args 4]
	     
	     if { $boton == "" } {
		return ""
	     }
	     if { $const_type == "Linear_Elastic" } {
		catch {grid remove $boton}
	     } else {
		catch {grid $boton}
	     }
	     return ""
	}
	"CLOSE" {

	}           
    }
}

proc MaterialWidget::Comando {} {
    variable const_type
    if {$const_type == "Linear_Elastic" } {
       return
    } else {
       GidOpenMaterials $const_type
    }
}